/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# Apple Macintosh Developer Technical Support
#
# MultiFinder-Aware SoundApp Application
#
# SoundAppSnds.r - MPW 3.0/3.1 Rez Source
#
# Jim Reekes - Macintosh Developer Technical Support
# Copyright  1989-1990 Apple Computer, Inc.
# All rights reserved.
#
# Versions:
# 			1.03					May, 1990
#
# Components:
# 			SoundApp.make		May 1, 1990			MPW build script
# 			SoundApp.p			May 1, 1990			Pascal source code
# 			SoundApp.r			May 1, 1990			Rez source code
# 			SoundAppSnds.r		May 1, 1990			Rez source code
# 			SoundUnit.p			May 1, 1990			Pascal source code
#
# Formatting was done with FONT = Monaco, SIZE = 9, TABS = 3
#
# SoundApp.p is a sample application source file for demonstrating the
# Sound Manager.  It requires the use of the SoundUnit to handle all of
# the sound routines.  This portion of the source code is the sources to
# the 'snd ' resources for Rez.  Its not what I would want to use
# continually as a snd compiler to say the least.  What I needed was an
# easy way to build snd resources that simple contained a sequence of
# sound commands.  I wanted to transcribed some given music, and didnt
# want to enter all the data using hex. I did build a ResEdit template,
# but even that was more gross that the method below.  I would really
# like to see a MIDI file to snd file exchange utility.  So heres what
# I came up with.
#
# I first had to define all the possible notes that could be used in a
# noteCmd.  These are the 127 MIDI note values.  There are 12 keys that
# cover 11 octaves.  By using the system below, I can define the key I
# want (such as D sharp) and then specify the octave (such as the octave
# that has D sharp next to middle C, or the 6th octave). This helped me
# to enter the notes by keeping in the proper key signature and range.
# I wouldnt use this system to enter any music beyond the 19th century.
#
# The next problem was to come up with a scheme for describing the
# durations of a note or rest.  The Sound Manager is based on 2000 beats
# per second.  Standard music is based on a number of beats per minute.
# Use the number of Sound Manager beats in a minute and divide by the
# number of beats you want per minute.  90 on the metronome is a typical
# tempo.  To get this tempo, use (kSMBeatsPerMinute / 90).  Disco is
# (kSMBeatsPerMinute / 120).
#
# The result of this calculation will give you the duration of a
# quarter note in 4/4 time.  If you wanted to play an eighth note, take
# the songs beat and divide it by 2.  To get a whole note, multiply by
# 4.  If youre still with me or interested in this process, youre a sick
# person.  I also found that putting the measure numbers in to the
# sequence helped to keep things straight while looking at the music
# manuscript.  If you want to change the tempo of one of the songs
# below, just change the #define that calculates the beat duration.
#
# Once I entered all the notes and got through a session with MPWs
# Rez, I had to compare my results with the original music.  I had good
# dictation drills while attending music courses at the university, so I
# was prepared for this next part.  I had to listen to the output of the
# Sound Manager and circle the incorrect note on the manuscript.  Go
# back to MPW and correct the Rez source and start over.  Ive found
# that restCmds dont cause the channel to be quiet unless preceded by a
# quietCmd.  A quietCmd has no duration. At least tempos can easily be
# adjusted.  If you want to transpose the key signature, you could write
# a sound channel modifier when the Sound Manager supports them.#
#
# Jim Reekes E.O., Macintosh Developer Technical Support
# Tuesday, January 30, 1990  1:01 PM
#
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// INCLUDES
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Types.r"
#include "SysTypes.r"

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// CONSTANTS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define kOctave1		0					/* octaves of MIDI note values */
#define kOctave2		12
#define kOctave3		24
#define kOctave4		36
#define kOctave5		48
#define kOctave6		60
#define kOctave7		72
#define kOctave8		84
#define kOctave9		96
#define kOctave10		108
#define kOctave11		120

#define Akey			-3					/* the note A */
#define Bbkey			-2					/* the note B flat */
#define Bkey			-1					/* the note B */
#define Ckey			0					/* the note C */
#define Dbkey			1					/* the note D flat */
#define Dkey			2					/* the note D */
#define Ebkey			3					/* the note D flat */
#define Ekey			4					/* the note E */
#define Fkey			5					/* the note F */
#define Gbkey			6					/* the note G flat */
#define Gkey			7					/* the note G */
#define Abkey			8					/* the note A flat */

#define kFullAmp		0xFF000000		/* loudest amplitude setting */
#define kSMBeatsPerMinute (2000	* 60)	/* Sound Mgr beats per minute*/

/* Using the constant above, I can define a tempo of 60 beats per minutes
	by using the following expressions: (kSMBeatsPerMinute / 60). */

// This will define the tempo for the rScaleSnd.
#define kScaleBeat	(kSMBeatsPerMinute / 240)

// This will define the tempo for the rMelodyPart1 - 4.
#define kMelodyBeat	(kSMBeatsPerMinute / 90)

// This will define the tempo for the rCounterPt1 - 4.
#define kBachBeat		(kSMBeatsPerMinute / 60)

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// RESOURCE ID NUMBERS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// snd resource must be in the range 8192 or greater
#define rMoofSound	9000				/* This is the about sound */
#define rScaleSnd		9001				/* snd containing a scale */
#define rMelodyPart1	9002				/* snd containing a melody */
#define rMelodyPart2	9003				/* snd containing the harmony */
#define rMelodyPart3	9004				/* snd containing the harmony */
#define rMelodyPart4	9005				/* snd containing the harmony */
#define rWaveHarmony	9006				/* snd containing waveTable for harmony */
#define rWaveMelody	9007				/* snd containing waveTable for melody */
#define rCounterPt1	9008				/* snd containing soprano part of counter point */
#define rCounterPt2	9009				/* snd containing alto part of counter point */
#define rCounterPt3	9010				/* snd containing tenor part of counter point */
#define rCounterPt4	9011				/* snd containing bass part of counter point */
#define rSopranoVox	9012				/* snd containing waveTable of soprano voice */
#define rAltoVox		9013				/* snd containing waveTable of alto voice */
#define rTenorVox		9014				/* snd containing waveTable of tenor voice */
#define rBassVox		9015				/* snd containing waveTable of bass voice */


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// RESOURCE DEFINITIONS
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This snd is used for holding the waveTable used in harmony sounds.
 Unfortunately, MPWs rIncludes doesnt properly define the complete snd
 resource format (who can blame them) and therefore defining a waveTable
 has to be done in the general data format.  The Sound Manager will re-sample
 a wave table to 512 bytes.  Realizing this, a wave table sounds best at
 512 or even 256 bytes. */

data 'snd ' (rWaveHarmony, "wave harmony", purgeable) {
	$"0001"						/* format 1 snd */
	$"0001"						/* number of synths to be installed */
	$"0003"						/* snth to be used */
	$"00000000"					/* init option for synth */
	$"0001"						/* number of sound cmds */
	$"803C 0100 00000014"	/* waveTableCmd with dataPtr bit, 256 bytes, offset 20 bytes */
									/* the wave table data */
	$"807F7C7A 78757371 706E6D6C 6B6A6A6B 6B6D6E71 73777A7E 82868B90 959A9FA4"
	$"A9AEB2B7 BBBFC1C5 C8CBCED1 D3D6D8DA DCDEE1E3 E5E7EAEC EEF0F3F5 F7F8FAFB"
	$"FDFEFEFE FFFEFEFD FCFAF8F6 F4F1EEEB E8E4E1DD D9D5D1CD C8C4BFBB B6B1ACA7"
	$"A29D9893 8E8A8682 7F7C7976 7472706F 6E6D6D6D 6D6E6F70 71737576 787A7D7F"
	$"80818386 888A8B8D 8F909192 93939393 9291908E 8C8A8784 817E7A76 726D6863"
	$"5E59544F 4A45413C 38332F2B 27231F1C 1815120F 0C0A0806 04030202 01020202"
	$"03050608 090B0D10 12141619 1B1D1F22 2426282A 2D2F3235 383B3F41 45494E52"
	$"575C6166 6B70757A 7E828689 8D8F9293 95959696 95949392 908F8D8B 88868481"
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This snd is used for holding the waveTable used in melody sounds. */

data 'snd ' (rWaveMelody, "wave melody", purgeable) {
	$"0001"						/* format 1 snd */
	$"0001"						/* number of synths to be installed */
	$"0003"						/* snth to be used */
	$"00000000"					/* init option for synth */
	$"0001"						/* number of sound cmds */
	$"803C 0100 00000014"	/* waveTableCmd with dataPtr bit, 256 bytes, offset 20 bytes */
									/* the wave table data */
	$"808A9399 9D9E9A93 897C6C5B 49392B20 19161920 2D3E526A 829BB3CA DDECF8FE"
	$"FFFBF3E7 D8C6B39F 8B796757 4A3F3630 2B282728 292B2E32 363C4248 5059626D"
	$"7884909C A8B4BFC9 D2D9DFE3 E6E8E8E6 E4E1DEDA D6D1CDC8 C4BFBAB5 B0AAA59E"
	$"98918A82 7B736B63 5B544C45 3E37312C 27221F1C 1B1B1D20 252C343E 49566372"
	$"808E9DAA B7C2CCD4 DBE0E3E5 E5E4E1DE D9D4CFC9 C2BBB4AC A59D958D 857E766F"
	$"68625B56 504B4641 3C38332F 2A26221F 1C1A1818 1A1D2127 2E37414C 5864707C"
	$"88939EA7 B0B8BEC4 CACED2D5 D7D8D9D8 D5D0CAC1 B6A99987 75614D3A 28190D05"
	$"01020814 23364D65 7E96AEC2 D3E0E7EA E7E0D5C7 B7A59484 776D6662 63676D76"
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This snd is used for holding the waveTable used as the soprano voice. */

data 'snd ' (rSopranoVox, "Soprano", purgeable) {
	$"0001"						/* format 1 snd */
	$"0001"						/* number of synths to be installed */
	$"0003"						/* snth to be used */
	$"00000000"					/* init option for synth */
	$"0001"						/* number of sound cmds */
	$"803C 0100 00000014"	/* waveTableCmd with dataPtr bit, 256 bytes, offset 20 bytes */
									/* the wave table data */
	$"80899198 9FA3A4A4 A39F9890 887E746B 635E5A57 575B606A 7481909F ADBCCBD7"
	$"E0E8EEEF EEE8E0D7 CABCAC9D 8D7E7266 5E575454 585F6773 808F9FAD BDCBD8E4"
	$"EEF6FBFE FFFEFBF7 F2EBE4DF D8D3CFCC CACACBCC D0D4DADF E6EBF0F6 FAFCFEFE"
	$"FCF8F3EC E4DACFC2 B5A7998B 7C6F6256 4B433B36 322F2F30 343A4048 525C6773"
	$"808D99A4 AEB8C0C6 CCD0D1D1 CECAC5BD B5AA9E91 84756759 4B3E3126 1C140D08"
	$"04020204 060A1015 1A21262C 30343536 3634312D 28211C15 0E090502 0102050A"
	$"121C2835 43536171 808D99A1 A8ACACA9 A29A8E82 73635444 36292018 12111218"
	$"20293544 5361707F 8C96A0A5 A9A9A6A2 9D958C82 78706861 5D5C5C5D 61686F77"
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This snd is used for holding the waveTable used as the alto voice. */

data 'snd ' (rAltoVox, "Alto", purgeable) {
	$"0001"						/* format 1 snd */
	$"0001"						/* number of synths to be installed */
	$"0003"						/* snth to be used */
	$"00000000"					/* init option for synth */
	$"0001"						/* number of sound cmds */
	$"803C 0100 00000014"	/* waveTableCmd with dataPtr bit, 256 bytes, offset 20 bytes */
									/* the wave table data */
	$"807A746E 69635D57 524C4742 3E3B3835 34333435 373A3E43 494E545B 61676C71"
	$"75787A7C 7C7B7976 736F6B66 625E5B58 56565759 5C61676E 77808A95 A0ACB7C2"
	$"CCD6DFE7 EEF4F8FB FEFFFFFE FDFBF9F6 F3F0ECE9 E6E3E0DD DAD7D4D1 CECAC6C2"
	$"BDB8B3AD A7A19A93 8D867F79 736D6863 5F5C5A58 5757585A 5C5F6367 6B70757B"
	$"80858B90 95999DA1 A4A6A8A9 A9A8A6A4 A19D9893 8D87817A 736D665F 59534D48"
	$"433E3A36 322F2C29 2623201D 1A171410 0D0A0705 03020101 0205080C 1219212A"
	$"343E4954 606B7680 8992999F A4A7A9AA AAA8A5A2 9E9A9591 8D8A8785 84848688"
	$"8B8F9499 9FA5ACB2 B7BDC2C6 C9CBCCCD CCCBC8C5 C2BEB9B4 AEA9A39D 97928C86"
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This snd is used for holding the waveTable used as the tenor voice. */

data 'snd ' (rTenorVox, "Tenor", purgeable) {
	$"0001"						/* format 1 snd */
	$"0001"						/* number of synths to be installed */
	$"0003"						/* snth to be used */
	$"00000000"					/* init option for synth */
	$"0001"						/* number of sound cmds */
	$"803C 0100 00000014"	/* waveTableCmd with dataPtr bit, 256 bytes, offset 20 bytes */
									/* the wave table data */
	$"8081807F 7B756D63 57493B2D 20150C05 02010206 0B111820 28313A44 4D57616A"
	$"737A8084 86878684 817D7873 6E69635E 58534D49 45434242 44474C52 59616A74"
	$"7F8B97A4 B1BDC8D1 D8DDE0E0 DFDDDAD8 D6D5D6D9 DCE1E6EB F0F3F6F7 F7F5F1EC"
	$"E6DFD7CF C6BFB7B1 ABA59F99 91897F73 66594B3F 352D2928 2B313943 4F5B6774"
	$"808C99A5 B1BDC7CF D5D8D7D3 CBC1B5A7 9A8D8177 6F67615B 554F4941 3A312921"
	$"1A140F0B 09090A0D 10151A1F 24272A2B 2A282623 21202023 282F3843 4F5C6975"
	$"818C969F A7AEB4B9 BCBEBEBD BBB7B3AD A8A29D97 928D8883 7F7C7A79 7A7C8086"
	$"8D969FA9 B3BCC6CF D8E0E8EF F5FAFEFF FEFBF4EB E0D3C5B7 A99D938B 8581807F"
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This snd is used for holding the waveTable used as the bass voice. */

data 'snd ' (rBassVox, "Bass", purgeable) {
	$"0001"						/* format 1 snd */
	$"0001"						/* number of synths to be installed */
	$"0003"						/* snth to be used */
	$"00000000"					/* init option for synth */
	$"0001"						/* number of sound cmds */
	$"803C 0100 00000014"	/* waveTableCmd with dataPtr bit, 256 bytes, offset 20 bytes */
									/* the wave table data */
	$"807B7671 6B655F58 50484038 3028211A 15110E0C 0D0E1216 1C232B33 3C454E56"
	$"5D646A6F 73767879 7A7B7B7C 7C7D7F82 85898E94 9BA3ABB3 BCC4CDD5 DDE4EBF0"
	$"F5F9FCFE FFFFFFFE FCFAF7F4 F0ECE8E3 DED9D3CE C8C2BCB6 B0A9A49E 98938E89"
	$"85817D79 76726F6C 6965625F 5B585451 4E4C4A48 4848484A 4D51555B 61687078"
	$"80889098 9FA5ABAF B3B6B8B8 B8B8B6B4 B2AFACA8 A5A19E9B 9794918E 8A87837F"
	$"7B77726D 68625C57 504A443E 38322D27 221D1814 100C0906 04020101 01020407"
	$"0B10151C 232B333C 444D555D 656C7277 7B7E8183 84848585 8687888A 8D91969C"
	$"A3AAB2BB C4CDD5DD E4EAEEF2 F3F4F2EF EBE6DFD8 D0C8C0B8 B0A8A19B 958F8A85"
};


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 a simple C major scale covering two octaves */

resource 'snd ' (rScaleSnd, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* [1] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave5 + Ckey)
		},
/* [2] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave5 + Dkey)
		},
/* [3] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave5 + Ekey)
		},
/* [4] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave5 + Fkey)
		},
/* [5] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave5 + Gkey)
		},
/* [6] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Akey)
		},
/* [7] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Bkey)
		},
/* [8] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Ckey)
		},
/* [9] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Dkey)
		},
/* [10] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Ekey)
		},
/* [11] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Fkey)
		},
/* [12] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave6 + Gkey)
		},
/* [13] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave7 + Akey)
		},
/* [14] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave7 + Bkey)
		},
/* [15] */ noData,
		noteCmd {
			500, kFullAmp + (kOctave7 + Ckey)
		}
	},
	{
		/* array DataTables: 0 elements */
	}

};


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The opening to Mussorgskys Promenade in four voices.  This is the main melody. */

resource 'snd ' (rMelodyPart1, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Abkey)
		},
/* [4] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Ebkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Abkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [7] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Abkey)
		},
/* [4] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Ebkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Abkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [7] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Ebkey)
		},
/* [8] */ noData,
		quietCmd {
		},
/* [9] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Fkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [7] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Abkey)
		},
/* [8] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Fkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [7] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Abkey)
		},
/* [8] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave5 + Ebkey)
		},
/* [9] */ noData,
		quietCmd {
		},
/* [10] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat * 3), kFullAmp + (kOctave5 + Ebkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat * 2), kFullAmp + (kOctave4 + Abkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat * 3), kFullAmp + (kOctave4 + Ebkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat * 2), kFullAmp + (kOctave3 + Abkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}

};


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The opening to Mussorgskys Promenade in four voices.  This is the first part
 of the supporting harmony. */

resource 'snd ' (rMelodyPart2, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 5)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 6)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Abkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Dbkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Fkey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Gkey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 5)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 6)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Abkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Dbkey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Bbkey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Abkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			(kMelodyBeat + (kMelodyBeat / 2)), kFullAmp + (kOctave6 + Ebkey)
		},
/* [5] */ noData,
		quietCmd {
		},
/* [6] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Dbkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Dbkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ end ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}

};


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The opening to Mussorgskys Promenade in four voices.  This is the second part
 of the supporting harmony. */

resource 'snd ' (rMelodyPart3, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 5)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 6)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Abkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Fkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Abkey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave8 + Bbkey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 5)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 6)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Dbkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Fkey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Ebkey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Dbkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave6 + Abkey)
		},
/* [6] */ noData,
		quietCmd {
		},
/* [7] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Dbkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}

};


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The opening to Mussorgskys Promenade in four voices.  This is the third part
 of the supporting harmony. */

resource 'snd ' (rMelodyPart4, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 5)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 6)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Abkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Abkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Fkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Ckey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave8 + Ebkey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 5)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		restCmd {
			(kMelodyBeat * 6)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Abkey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Dbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Abkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave8 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Gkey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Abkey)
		},
/* [4] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Ebkey)
		},
/* [6] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Ckey)
		},
/* [7] */ noData,
		quietCmd {
		},
/* [8] */ noData,
		restCmd {
			(kMelodyBeat / 2)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			(kMelodyBeat / 2), kFullAmp + (kOctave7 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Abkey)
		},
/* [5] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [7] */ noData,
		noteCmd {
			kMelodyBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}
};


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The song Schaut, ihr Snder arranged by JS Bach.  This is the soprano part. */

resource 'snd ' (rCounterPt1, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Akey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Akey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave7 + Akey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave7 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Akey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Ckey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gkey)
		},
/* [6] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave7 + Akey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Akey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The song Schaut, ihr Snder arranged by JS Bach.  This is the alto part. */

resource 'snd ' (rCounterPt2, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gbkey)
		},
/* [6] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gbkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave7 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave7 + Akey)
		},
/* [6] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Akey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat + (kBachBeat / 2)), kFullAmp + (kOctave6 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Gbkey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat + (kBachBeat / 2)), kFullAmp + (kOctave6 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 4), kFullAmp + (kOctave6 + Fkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 4), kFullAmp + (kOctave6 + Ekey)
		},
/* [6] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ekey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Ckey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave7 + Akey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave7 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Abkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 12 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gbkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The song Schaut, ihr Snder arranged by JS Bach.  This is the tenor part. */

resource 'snd ' (rCounterPt3, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Akey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat + (kBachBeat / 2)), kFullAmp + (kOctave6 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave5 + Gbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat + (kBachBeat / 2)), kFullAmp + (kOctave6 + Akey)
		},
/* [6] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Akey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Ckey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Akey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ebkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Fkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Ekey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Ckey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat + (kBachBeat / 2)), kFullAmp + (kOctave6 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave6 + Bkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}
};

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The song Schaut, ihr Snder arranged by JS Bach.  This is the bass part. */

resource 'snd ' (rCounterPt4, purgeable) {
	FormatOne {
		{
			/* array Synthesizers: 0 elements */
		}
	},
	{
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 1 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat + (kBachBeat / 2)), kFullAmp + (kOctave5 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Gbkey)
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave4 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 2 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave4 + Gkey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Akey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Ckey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 3 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave5 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 4 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Akey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Bbkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 5 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Bbkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Abkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* [6] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 6 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ekey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave5 + Bbkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 7 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Bbkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave6 + Bbkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave6 + Akey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave4 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 8 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ekey)
		},
/* [4] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Fkey)
		},
/* [5] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Dkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 9 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ckey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ckey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave4 + Fkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 10 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Gkey)
		},
/* [2] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Fkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Ebkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Dkey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Bbkey)
		},
/* [6] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ebkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		quietCmd {
		},
/* [2] */ noData,
		restCmd {
			kBachBeat
		},
/* [3] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Bkey)
		},
/* [4] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Ckey)
		},
/* [5] */ noData,
		noteCmd {
			(kBachBeat / 2), kFullAmp + (kOctave5 + Dkey)
		},
/* [6] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Ekey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ measure 11 ~~~~~~~~~~~~~~~~~~~~~~ */
/* [1] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave5 + Dkey)
		},
/* [2] */ noData,
		noteCmd {
			kBachBeat, kFullAmp + (kOctave4 + Dkey)
		},
/* [3] */ noData,
		noteCmd {
			(kBachBeat * 2), kFullAmp + (kOctave4 + Gkey)
		},
/* ~~~~~~~~~~~~~~~~~~~~~~ END ~~~~~~~~~~~~~~~~~~~~~~ */
	},
	{
		/* array DataTables: 0 elements */
	}
};


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 This is the sampled sound used in the about window. */

resource 'snd ' (rMoofSound, "Moof", purgeable) {
	FormatOne {
		{generalSampledSynth, 0}
	},
	{hasData,bufferCmd {20}		/* buffer data offset is 20 bytes */
	},
	{27616,							/* number of samples */
	 0x56EE8BA3,					/* sample rate */
	 0,								/* loop starting point */
	 27615,							/* loop ending point */
	 60,								/* base note */
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80807F7F"
		$"7F7F7F7F 7F7F8080 80808080 80808080 80808080 7F7F7F7F 7F7F7F7F 7F7F7F7F"
		$"7F7F7F7F 7F7F7F7F 7F7F8080 80808080 80808080 80808080 7F7E7D7D 7C7B7A7A"
		$"79787674 72727272 74767878 7877787A 7C7D7D7D 7D7F7F7F 7F7F7F7F 80828383"
		$"83838282 82838587 89898A8A 89878686 87898989 88868484 84848484 83838282"
		$"817F7E7D 7D7E7F7F 7F7F7E7C 7B797A7C 7D7D7E7E 7D7D7D7D 7D7F8080 80808080"
		$"80808080 80808080 7F7E7E7E 7F7F7F7F 7F7F7F7F 7F7F7F7F 7F808080 7F7F7F7F"
		$"80808080 80807F7F 7F7F8080 80808080 80808080 80808181 81818181 80808080"
		$"81818181 81818181 81818181 81818181 82828282 83838383 82828282 83838382"
		$"82828282 82828282 82828181 807F7F7F 7F7F8080 7F7F7F7F 7F7F8080 81818181"
		$"80808081 82828181 81818180 7F7F7F7F 80807F7F 7E7E7E7E 7F7F7F7F 7F7F7F7F"
		$"7F7F7F7F 7F7F7F7F 7F7F8080 80807F7F 7E7E7D7C 7B7A7978 77757372 71717274"
		$"75777776 7677797C 7D7B7C7E 7F7F7F7F 7F808181 82848484 83828282 83858688"
		$"88888786 85848587 88888886 84848382 82828383 82818080 7F7D7D7D 7D7D7D7E"
		$"7E7D7B79 787A7B7C 7D7D7C7B 7B7B7B7D 7E7F8080 807F7F7F 7F7F7F7F 8080807F"
		$"7F7F7F7F 80808080 80807F7F 7F7F7F80 81818080 80808080 80808181 807F7F7F"
		$"80808080 81818080 80808080 81818181 81818181 80808081 82828181 80808080"
		$"81818282 81818181 81818181 81818282 82828282 83838484 83838281 81818181"
		$"81807F7F 7F7F7F7F 7E7E7E7E 7F7F7F7F 7F808181 82828181 81818282 81818181"
		$"81807F7F 7F7F7F7F 7F7F7F7E 7E7E7F7F 7F7E7E7E 7F7F7F7F 7F7F7F7F 7F7F8080"
		$"80808080 7F7F7E7C 7A787878 77757371 706F7072 75777776 77797B7C 7C7B7C7E"
		$"80808181 81818080 81848686 85848382 82838587 88898988 87858486 87888887"
		$"86868583 82828282 82828282 807E7D7D 7D7D7D7E 7E7D7B79 7878797B 7C7D7E7E"
		$"7D7C7C7D 7E7F8080 81818080 7F7F7F7F 80828281 80807F7F 7F7F8080 80807F7E"
		$"7E7E7E7F 80818180 7F7F7F7F 7F808181 80808080 7F7F7F7F 80818181 80808081"
		$"82828282 82828181 81818182 83838383 82808081 82828282 82828282 82828383"
		$"83838383 82828282 82828282 83838383 82828282 83838281 80807F7F 7F7F7F80"
		$"80808080 7F7E7E7F 80808080 80818181 80808081 82828181 807F7F7F 7F7F8080"
		$"7F7F7F7F 7F7F7F7F 7F7F8080 7F7F7F7F 7F7F8080 80808080 7F7F7E7E 7D7C7B7A"
		$"79777573 71707070 71737575 76767678 7A7C7C7B 7B7D7E7F 80808181 81818385"
		$"85848484 84848485 86888989 88878685 85878888 88868585 84838383 82828181"
		$"8181807E 7D7D7D7D 7D7D7D7C 7B797878 797B7C7C 7C7C7C7C 7C7E7F7F 7F808181"
		$"80808080 81818182 82828180 80808080 80808080 7F7F7E7E 7E7E7F80 807F7F7F"
		$"7F7F7F7F 8080807F 7F7F8080 7F7F7F80 80808080 80808080 81818282 82828181"
		$"81828383 82828181 81818181 81818181 81818181 81828383 84848383 82828282"
		$"82828282 82828282 82828282 81808080 7F7F7F80 80807F7F 7E7E7E7E 7F7F8080"
		$"7F7F7F7F 80808080 80807F7F 7F7F7F7F 7F7F7F7F 7E7E7E7E 7E7E7F7F 7E7E7E7E"
		$"7E7E7E7E 7E7F8080 7F7F7F7F 7F808181 807F7E7C 7B797878 77777573 716F6F6F"
		$"71747676 76767779 7B7B7B7B 7D7F8080 7F7F7F7F 7F818385 85858482 82828486"
		$"888A8A8A 89878585 86888989 88868583 82828282 83838383 82807F7E 7E7E7F7F"
		$"8080807E 7C7A797B 7C7E7F7F 7E7C7C7C 7D7F8081 82828181 80808080 80828282"
		$"81808080 7F7E7E7F 8080807F 7E7E7E7E 7E808181 807F7F7F 7F7F8080 80808080"
		$"7F7F7F7F 7F7F8080 80807F7F 7F7F8080 81818181 80808080 80818181 80808080"
		$"80808181 81818181 81818181 82828282 83838282 82828282 82828282 81818181"
		$"81828282 81808080 80808080 7F7F7F7F 7F7F7E7E 7E7E7E7E 7F7F8081 81808080"
		$"80808080 7F7F7E7E 7E7E7F7F 80808080 81818181 81818181 82828282 81818080"
		$"80808181 807F7E7E 7D7D7C7A 79797878 77757371 70707173 75777777 7777797B"
		$"7C7C7D7F 80807F7F 7F808182 83858585 84848383 83858789 8B8B8A88 87858586"
		$"888A8A88 86848281 81818282 82818080 7F7D7D7D 7D7D7E7F 7F7D7C7A 79797A7C"
		$"7D7D7D7C 7B7B7B7C 7D7F8080 80807F7E 7E7F8080 80818180 7F7F7F7F 7F7F7F7F"
		$"7F7F7E7E 7E7E7E7E 7F80807F 7F7F7F7F 7F7F8080 807F7F7F 7F7F7F7F 7F808080"
		$"80808080 80808181 80808080 80808081 82828282 81818181 81818282 82828282"
		$"82828283 84848585 84848484 84848484 84848383 82828282 82828180 7F7F7F7F"
		$"7F808080 80807F7F 7F7F7F7F 7F7F8080 80808181 81818181 81818080 80808080"
		$"80808080 80808081 81808080 81818181 80808080 81818181 807F7E7E 7D7D7C7A"
		$"79787777 76757371 6F6F6F70 72757675 7576787A 7B7A7A7C 7D7F7F7E 7E7F7F7F"
		$"80828484 84848383 83848688 898A8B8B 89878686 87888989 88868482 82828282"
		$"82828282 817F7E7D 7D7D7E7E 7F7F7E7C 7A79797B 7C7D7E7E 7D7C7C7C 7C7E7F7F"
		$"80808080 7F7F7F7F 7F808181 807F7F7F 7E7E7E7E 7F7F7F7F 7E7D7D7D 7E808181"
		$"807F7F7F 80808080 7F7F7F7F 80807F7F 7F7F8080 80808080 80808080 81818180"
		$"80808080 81818282 81818181 81818282 82828282 82828283 84848484 84848383"
		$"83838383 83838484 83828282 82828282 81808080 80808080 80808080 7F7F7E7E"
		$"7E7F8080 7F7F7F81 81808080 80808080 80808181 81818181 81818181 81818080"
		$"80808181 80807F7F 7F808080 80807F7F 7E7E7D7B 79787777 76767471 6F6F6F6F"
		$"70747575 74747578 7A7A7B7B 7C7E7E7E 7E7E7F7F 7F808284 84848382 82828486"
		$"888A8B8B 89878686 86888989 89878584 83838383 83838383 82807F7D 7D7D7E7E"
		$"7E807F7D 7B797878 797B7C7C 7C7B7A7A 7A7C7D7E 7F7F7F7F 7F7F7F7F 7F7F8081"
		$"81818080 7F7E7E7E 7F7F7F7F 7E7E7E7E 7E7F8080 807F7F7F 7F7F7F80 80807F7F"
		$"7F7F7F7F 7F7F7F7F 80808080 80808080 81818282 81818181 81818282 82818181"
		$"81818282 83838282 82828282 83838383 84848383 83838484 83838383 83828282"
		$"82828181 80807F7F 7E7E7E7E 7E7E7E7E 7F7F7E7D 7D7E7F7F 7E7E7E7F 80808080"
		$"81818181 81818181 80808080 80808080 80808181 807F7F7F 7F7F8080 81818080"
		$"80807F7F 7F7F7E7E 7D7B7977 76767574 73716F6E 6E707274 75757677 797B7C7A"
		$"797B7C7E 7F7F7F7F 7F7E7F81 83848585 84848484 8587888A 8A8A8987 86868789"
		$"89888786 85838281 81818181 82828280 7E7D7D7D 7D7F8080 7E7C7A78 78797A7C"
		$"7D7D7D7B 7B7B7C7E 7F7F7F80 80808080 7F7F7F80 81818180 7F7F7E7E 7E7E7F7F"
		$"7F7F7E7E 7E7E7E80 81818080 80808080 80808181 81818181 80808080 81818181"
		$"81818181 81818282 81818181 81818181 82828282 82828282 82828282 82828282"
		$"82828383 83838484 84848484 84848484 83838282 82828282 81807F7F 7F7F7E7E"
		$"7E7E7E7E 7E7E7E7E 7E7E7E7E 7E7E7E7E 7F7F7F7F 7F7F7F80 81818181 81818181"
		$"82828281 81818282 82828180 80808181 81818080 80808080 81818282 82828181"
		$"80807F7E 7D7B7A79 77757371 6F6F7072 74767674 7476787A 7A7A7B7D 7E7E7E7E"
		$"7E7F8080 81838484 83838282 83858789 8A8A8987 86858586 87888886 85848382"
		$"82828383 83838282 817F7F7F 7F7F8080 80807E7C 7A7A7A7B 7C7D7D7C 7B7A7A7B"
		$"7C7E7F7F 7F7F7E7E 7E7E7F7F 7F808080 7F7F7E7E 7E7E7E7E 7F7F7E7E 7E7E7E7E"
		$"7E808080 7F7F7F7F 7F7F8080 807F7F7F 80807F7F 7F7F7F7F 7F7F7F7F 80808080"
		$"81818181 81818181 82828282 81818080 80818181 81818181 81818181 82828383"
		$"84848383 83838484 84848383 82828282 81818080 80807F7F 7F7F7F7F 7F7F7F7F"
		$"7E7E7E7E 7E7F8080 80808080 7F7D7D7E 7F7F8080 80808080 81818282 82828282"
		$"82828282 81818181 82828181 80808080 81818181 81808080 80807F7F 7E7C7B79"
		$"78777572 706E6D6D 6F717374 74737476 78797A7A 7B7D7D7C 7C7C7D7D 7E7F8082"
		$"83838282 82828385 87898B8B 8A888786 8688898A 8A888684 82828282 82828282"
		$"8181807F 7F7F8080 81828280 7E7C7A7A 7A7C7D7D 7D7C7B7A 7A7C7D7E 7F7F7F7F"
		$"7F7F7F7F 7F7F8081 81818080 7F7F7E7E 7E7E7F7F 7F7E7D7D 7D7D7E7F 80807F7F"
		$"7F7F8080 81818080 80808080 7F7F7F7F 7F7F7F7F 7F7F7F7F 7F7F8080 80808080"
		$"80808181 82828180 80808181 82828282 82828282 83838484 84848585 84848484"
		$"84848484 85858482 81818181 81818080 7F7F7F7F 7F7F7F7F 7F7F7F7F 7E7E7E7E"
		$"7F7F8082 83838281 81828383 84848484 83828282 82828383 83838281 80808080"
		$"80808181 81818181 81818182 83838280 7F7D7B79 79797878 76726F6D 6C6C6D71"
		$"73737373 74767779 7A7A7B7D 7F7F7E7E 7D7D7D7F 81848584 83818080 81838689"
		$"8B8B8A88 86858587 88898987 86848281 81818282 82828281 807E7D7D 7E7F8080"
		$"807F7D7A 7979797B 7C7D7D7C 7B79797B 7C7D7E7E 7F7F7F7F 7F7F7F7F 7F808181"
		$"81807F7F 7F7F7F7F 80807F7F 7E7D7D7D 7E808080 7F7F7F7F 80808080 80808080"
		$"80808080 80808080 7F7F7F7F 7F7F7F80 81818181 80808080 81818282 82828181"
		$"81818282 81818181 82828282 83838484 85858484 84848484 85858484 83828181"
		$"81818181 80808080 7F7F7F7F 80808080 807F7F7F 7F7F8080 81828282 81818181"
		$"81818282 82828181 81818080 7F7F7F7F 80807F7F 7F7F8081 82828282 82828282"
		$"81807F7D 7B797776 75747371 6F6D6B6B 6C6E7173 74747476 787A7B7A 7B7D7E7F"
		$"7F7F7F7F 7F7F8082 84848483 82818183 85888A8B 8B898886 85868789 89878686"
		$"85838282 82828383 83848482 807F7F7F 7F818181 7F7C7A78 77797A7C 7D7D7C7A"
		$"79797B7D 7E7E7E7F 7F7F7F7F 7F7F7F80 81828281 80807F7D 7D7D7E7F 80807F7E"
		$"7D7D7D7F 80808080 80808080 81818080 80808080 80808080 80808080 80808181"
		$"81818181 82828282 81818182 82828282 81818181 82828282 82828282 82828282"
		$"83838383 82828282 82828282 83838383 82828282 82828282 81808080 80807F7F"
		$"7F7F7F7F 80807F7F 7F7F8081 82828281 81818282 81808080 80808181 80807F7F"
		$"7F7F8080 81818181 81818181 82828383 8282817F 7E7C7A78 76767575 73706D6B"
		$"6A6A6D71 73737373 74767778 797B7D7F 80808080 80808081 83858686 85848382"
		$"8385878A 8C8C8B89 87858587 898B8B89 88868483 83838484 84848484 83817F7E"
		$"7E7E7F80 80807E7B 79787879 7B7D7D7B 7A78787A 7B7B7C7C 7D7D7E7E 7E7E7F7F"
		$"7F7F8080 80807F7F 7E7D7D7D 7D7D7E7E 7E7D7D7D 7E7F8080 80808080 80808181"
		$"81808080 80808080 80807F7F 7F7F8080 80808080 80808181 80808080 80818282"
		$"81818080 80808181 82828282 82828282 83838383 84848484 84848383 83838484"
		$"84838282 82828282 81818080 80808080 7F7D7D7D 7D7E7F7F 80807F7F 7F818283"
		$"84848484 83838383 82828282 82828281 807F7F80 81818282 81808080 80818283"
		$"84848381 807E7C7B 7A787775 7472716F 6D6B6A6A 6B6F7172 72727375 77797A7A"
		$"7B7D7E7E 7F7F7F7F 80808183 84848383 83838385 87898A8B 8B8A8988 8888898B"
		$"8B898886 85848484 84848484 84848482 81818080 80818181 807E7C7A 79797A7B"
		$"7C7C7B79 78797A7C 7C7C7C7C 7D7D7E7F 7F7F7F7F 7F7F8080 7F7F7E7E 7E7E7E7E"
		$"7F7F7E7E 7D7D7D7E 7F808080 80808080 80818180 80808080 80807F7F 7F7F7F7F"
		$"80808080 81818181 81818181 80808080 81818282 81818080 80808181 82828282"
		$"82828282 83838383 83838383 83838281 81828383 82828282 82828282 82828181"
		$"80807F7F 7F7F8080 80808181 82828282 83848484 84848383 82828282 82828181"
		$"81818080 80808080 81818181 80808080 81818181 807F7E7C 7A787676 7573706C"
		$"69686869 6C707170 70707377 7878797B 7D7F7F7F 7F80807E 7F828486 86848382"
		$"82828486 888A8B8B 8A888787 888A8B8B 8A888785 84848585 84848485 85838280"
		$"80808182 83848381 7E7B7A7A 7A7B7C7D 7D7B7978 797B7C7C 7C7C7D7D 7D7D7D7E"
		$"7E7E7E80 81818180 7F7D7C7C 7C7D7E7E 7E7E7D7B 7B7C7D7E 7F7F7F7F 7F7F8080"
		$"80808080 80808181 807F7F7F 7F7F8080 80808080 80808181 82828180 80808081"
		$"81818181 81818181 81818181 81818181 81818282 82828282 82828383 83838383"
		$"84848484 83838383 84848484 83828181 80808080 80808181 81818182 83838485"
		$"86868585 84848484 84848484 83818080 80818181 81818080 80808181 82828282"
		$"817F7E7C 7A787776 74726E6A 67666667 6A6E6F6F 6F6F7074 7676777B 7D7D7E7E"
		$"7F7F7F7F 81858787 86858482 82838588 8A8B8B8A 89878686 87898B8B 8B898784"
		$"83838485 86868686 86848281 81818181 82838381 7E7A7979 79797A7B 7B797876"
		$"77797A7B 7C7C7C7B 7B7B7C7C 7C7C7C7E 7F7F7F7F 7E7D7C7C 7C7D7E7E 7E7E7D7C"
		$"7C7C7D7E 7F7F8080 7F7F7F80 81818181 81818282 81818080 80808181 81818181"
		$"81818181 82828281 81818282 83838282 81808080 81818181 80807F7F 7F7F7F7F"
		$"80808181 81818282 82828383 84848383 82828282 83838381 81818080 80808080"
		$"80818282 82828282 83838383 84848484 84848383 83838383 83818181 82828181"
		$"81818282 82838382 81818080 7F7E7D7B 79777573 706C6967 6666676A 6C6C6D6D"
		$"6E707273 7476787A 7C7C7D7F 80808284 85878786 85848383 83858789 8A8A8987"
		$"86868688 898B8B8B 8A888685 85868787 87888887 86848383 83838384 8484827E"
		$"7C7A7A7A 7B7C7C7C 7B79797A 7B7B7C7C 7C7C7C7C 7C7C7C7C 7C7D7E7F 7F7F7F7F"
		$"7E7C7C7C 7C7D7E7E 7D7D7C7C 7C7D7E7F 80808080 80808181 81818181 81818282"
		$"81818181 81818282 83838383 83838383 83838282 82828383 83838282 81818181"
		$"81818181 807F7F7F 7F7F7F7F 80808181 80808080 81818181 82828281 81818181"
		$"81818080 80807F7F 7F7F7F7F 80818181 81818181 82828384 85858585 85858484"
		$"84848484 84848484 83828282 82828383 83838282 8282817F 7E7C7B79 7775726F"
		$"6C686665 6668696B 6B6B6B6C 6E707273 7577797A 7B7D7E7F 80808285 86868585"
		$"84828283 85888A8A 8A8A8987 8787898B 8D8D8C8A 89878787 87878787 88888785"
		$"83838282 82838484 83817E7C 7A7A7A7B 7C7C7C7A 7979797A 7B7B7B7B 7B7B7B7B"
		$"7B7B7B7B 7B7C7D7D 7E7E7D7C 7B7B7B7B 7C7C7C7C 7C7C7C7C 7D7D7E7E 7F7F7F7F"
		$"80808181 81818181 82828282 82828282 82828383 84848484 84848383 82828282"
		$"83838484 83828282 82828181 8181807F 7E7E7E7E 7E7E7F7F 80807F7F 7F7F7F7F"
		$"80818282 81808080 80808081 81808080 7F7F7F7F 80808181 82828282 83838383"
		$"84868787 86868686 86868686 85858585 85858585 85858585 85858686 86858483"
		$"82807F7D 7B797774 706C6866 64646568 6A6A6969 696B6C6E 70727577 787A7B7B"
		$"7B7B7D81 83858585 84828181 8285888A 8B8B8B89 8787888A 8C8E8E8D 8B898787"
		$"87878888 88898988 87858585 85858686 86858381 7E7C7B7B 7C7D7D7D 7C7A7A7A"
		$"7A7B7C7C 7C7C7C7C 7B7B7B7B 7B7B7C7D 7E7E7D7D 7C7C7B7A 7A7B7C7C 7C7C7B7B"
		$"7B7B7C7D 7E7E7E7E 7F7F7F7F 80808181 81818282 82828282 82828383 84848484"
		$"84848484 85858484 84848484 84848382 81818181 81818181 80807F7F 7F7F7F80"
		$"81818181 81818080 80818282 82828281 81818181 81818080 80808080 7F7F7F7F"
		$"7F808181 82828282 82828384 85858585 85858585 85858686 86868585 84848484"
		$"84848586 87878685 84838280 7E7C7A78 75716C68 65636466 68686867 67686A6C"
		$"6E6F7072 74777979 7A7A7B7D 7F818282 8281807E 7F818385 86888887 86848587"
		$"898B8C8E 8E8C8A88 88898989 898A8B8B 8A888888 87878789 89898886 83807E7D"
		$"7D7E7F7F 7F7D7C7B 7B7C7D7D 7D7C7C7C 7C7C7B7B 7B7B7B7B 7C7C7C7B 7A7A7978"
		$"78797A7A 7A7A7979 79797A7A 7B7B7C7C 7C7C7C7D 7E7E7F7F 7F808181 81818181"
		$"81818182 83838383 84848484 85858483 83838484 85858584 83838282 81818181"
		$"80807F7F 7F7F7F7F 80818181 81818181 80808080 81818282 81807F7F 7F7F8080"
		$"80807F7E 7E7E7E7E 7F7F8080 80808080 81818183 84858686 85858585 85858686"
		$"86868685 85858585 86868788 89898888 87858482 7F7D7A76 726D6967 6666686A"
		$"6A686868 696B6D6F 70727476 7778797A 7B7C7E80 82828180 7F7E7E80 82848688"
		$"88878685 8586888A 8B8D8C8A 88878787 88888989 89898887 87878888 898A8B8B"
		$"8A888683 817F7F7F 80807F7E 7D7B7A7B 7C7D7D7D 7C7C7C7C 7C7C7B7B 7B7C7D7D"
		$"7C7C7B7B 7A797979 79797A7A 79787878 7878797A 7B7B7A7A 7A7B7C7D 7E7E7F7F"
		$"80808080 80808080 81818283 84848484 84858686 86868686 86868787 87878686"
		$"85858484 83838282 81818181 81818282 82828181 81818282 82828282 83838282"
		$"81808080 80807F7F 7E7E7E7E 7E7E7E7E 7E7E7D7D 7D7D7E7F 80818282 83838484"
		$"83838383 83848585 85848484 85858687 888A8A8A 89898886 8583817F 7B76716C"
		$"69676769 6A6B6B69 696A6B6D 6E707274 76787979 797A7C7E 80828484 82807E7D"
		$"7E808385 87888886 84848486 888A8C8C 8A888686 86868787 88888887 86868686"
		$"8688898B 8B8A8987 84828080 80807F7F 7E7D7C7B 7B7D7E7E 7E7E7D7D 7D7D7C7C"
		$"7C7C7C7D 7E7E7E7D 7C7B7A7A 7A7A7A7A 7A7A7978 77777778 79797A7A 7A7A7A7A"
		$"7B7C7D7D 7E7E7F7F 7E7E7E7E 7E7E7F7F 80818282 82828283 84848484 84848485"
		$"86868686 85848484 84848383 82828181 81818282 83838383 83838383 83838383"
		$"83838484 83818182 82828180 7F7F7F7F 7E7D7D7D 7D7D7D7D 7C7C7C7C 7D7D7E7F"
		$"80818181 81828383 83838383 83838484 84848585 85858687 88888989 89878684"
		$"817F7B76 716C6967 67696A6A 69676668 6A6D6F71 72737577 78797A7C 7D7F8183"
		$"83838280 80808183 85878888 87858484 8587898B 8C8C8A86 85858586 87888888"
		$"87858484 84858688 89898989 88868381 80808080 80807F7D 7B7B7B7D 7E7F7F7E"
		$"7D7C7C7C 7D7D7D7D 7E7E7E7E 7E7D7C7C 7B7A7A7A 7B7B7B7A 79787878 7878797A"
		$"7A7A7A7A 7A7B7C7C 7D7E7F7F 7F7F7E7E 7E7E7E7E 7E7F8081 82828282 83838484"
		$"85858585 85868787 86868686 86868585 85858483 82828283 84848585 85848383"
		$"83838384 84848383 82828282 82828282 81807F7F 7E7E7E7F 80807F7E 7D7D7D7D"
		$"7D7D7E7E 7F7F7F7F 80808080 81818282 83838383 83838383 84848585 86878889"
		$"8A8A8988 8785827F 7B75706B 6868686A 6B6B6A68 68696B6D 6E707173 75777878"
		$"7A7C7F81 82848482 817F7F7F 81838587 88888785 83838486 898B8C8A 88868585"
		$"85878888 88878684 84848486 888A8B8B 8A8A8886 84828080 80807F7F 7E7C7C7C"
		$"7C7E7F7F 7F7E7D7C 7C7C7C7C 7D7D7E7E 7E7E7D7D 7C7B7A7A 7A7A7A7A 7A7A7978"
		$"78787878 79797A7A 7A7A7A7B 7C7D7E7E 7F7F7F7F 7E7E7E7E 7E7E7F7F 80818282"
		$"83838484 85858585 85858585 86868787 86868686 85858484 83838282 82828383"
		$"83838383 82828282 82828383 84848382 81818181 82828281 807F7F7F 7F7F7E7E"
		$"7D7D7D7D 7C7C7C7C 7C7D7E7F 80808080 80818282 82828282 83838383 83838484"
		$"85868787 88898989 88878684 817E7A75 706A6765 66686969 69686868 696B6D6F"
		$"71737577 7979797B 7D7F8284 84848381 80808183 85878989 88868483 8486888B"
		$"8C8C8A87 85848486 87888989 88868585 8586888A 8B8B8B8B 89878583 81818181"
		$"81807F7E 7D7D7D7E 7F7F7F7E 7D7C7C7C 7C7C7C7C 7D7D7E7E 7D7D7C7B 7A7A7A7A"
		$"7A7A7A7A 79787777 77787979 7A7A7A7A 7A7B7C7C 7D7E7F7F 7F7F7E7E 7E7E7E7E"
		$"7F7F8081 82828383 84848585 85858686 86868787 87878686 85858585 84848383"
		$"82828282 83838484 84828181 80808080 80808080 7F7F7F7F 80808181 80807F7F"
		$"7F7F7F7F 7E7E7D7D 7C7C7C7C 7D7E7F80 81838485 86868787 87878787 86868686"
		$"85858484 84848587 898B8D8E 8E8E8C8A 8784807C 766E6458 504C4C50 545A5D5D"
		$"5E606367 6C727678 7A7A7A78 77757576 797D8082 84848585 878C9299 9EA09E99"
		$"948E8B8D 90959898 958D8681 8082868C 90929291 908E8C8C 8C8E8F91 908E8983"
		$"7C76716F 6E707172 72706F6F 72767A80 83848381 7E7A7876 75747474 75757575"
		$"75757779 7C808385 85848381 81828486 8583807C 79777779 7B7F8181 81807F7E"
		$"7E808285 87898987 84828080 80828280 7E7C7977 76787A7E 81838383 83838385"
		$"87898988 8683807D 7C7C7C7E 7F7F7F7E 7D7D7E80 82848687 87878685 84838282"
		$"82828180 7E7B7979 7A7C7F81 83838280 80808284 87898A8B 8B8A8989 88878787"
		$"88888888 87858484 8587898B 8D8E8E8E 8D8D8D8D 8C8A8681 7A726759 4D413A39"
		$"3D464F57 5B5B5C5F 656D747A 7D7D7B78 7573716F 71757A80 85898C8D 8E8E9197"
		$"9DA4A8A8 A49C948D 8A8C8F95 9898948C 85818083 878D9092 9291908E 8D8D8D8F"
		$"9193928E 887E756D 6867686B 6D6D6C6A 6A6E737B 82898C8B 88827C76 716D6C6C"
		$"6C6D6E6E 6F6F7174 787E8389 8C8C8B88 86848384 8585837F 7B777575 787C7F83"
		$"84848383 83838486 888A8B8B 89878583 83838484 82807D79 76767779 7B7E8082"
		$"83838485 87898B8B 8A86827C 78747273 75787A7C 7D7E7F80 82868A8E 9091908C"
		$"88827F7D 7C7E7F7F 7D7B7978 797B7F83 86898A89 8783807E 7E818488 8A8B8A87"
		$"85838486 898D8E8C 89858280 8185888C 8F919393 92919090 8F8D8880 74635240"
		$"342E313D 49565E60 605F6268 707A8082 7F79736D 6967686D 747C8389 8C8D8E8F"
		$"9399A0A9 AEAEA99F 958B8687 8C959B9D 9990877F 7C7F858D 9294938F 8C888788"
		$"8B909497 97948D83 796F6A69 6A6C6D6C 69656365 6B76818C 92949087 7E746E6A"
		$"69696A6A 6A696868 6A6E747C 838A8E8E 8D898582 82848688 86817B75 7171747A"
		$"7F858888 86838181 8284878A 8C8C8B89 86848383 84858583 7F7A7672 71727579"
		$"7C7F8182 8384868A 8C8D8C89 847E7872 6F6F7074 777B7D7E 7F808387 8C929595"
		$"928C8783 80808080 7F7D7C7A 79797A7E 8286898C 8D8B8781 7C7A7A7D 81858888"
		$"87858484 868A8D8F 8E8C8883 80808184 888C8E8F 90909191 9191908E 8A837765"
		$"523E312B 2E3C4A58 5F60605F 626A727C 81817E78 726C6764 6569707A 828A8E8E"
		$"8F8F939B A4AEB4B4 ADA19488 83878E99 A0A29E94 8A817E82 878F9393 928E8A87"
		$"86878A90 95999995 8E82786E 6868696C 6D6B6763 61636A76 818C9294 90887F75"
		$"6D696767 67686969 69696B6F 757D848B 8F8F8D88 84807F81 8284827E 79737171"
		$"757B8084 87878785 83818182 84878A8C 8C8A8785 84858687 8684807A 76727173"
		$"767A7D7F 80808284 878B8D8D 8B868078 736F6E70 7275787A 7C7E8287 8C929698"
		$"97938F8A 86827F7E 7E7E7E7E 7D7C7C7D 7F83878C 8F8F8C88 837E7B7B 7C7E8082"
		$"83838484 8587888A 8B8B8B89 86838180 8184888C 91959898 9795918C 857B6E5C"
		$"4B3B3231 36424C54 5A5E6268 6E767B7D 7D7A7773 6E6A6662 63676E78 80878C8E"
		$"909298A1 A9B0B2AF A79B928C 8B90969C 9D9A948C 8785878C 90939492 8F8B8886"
		$"87898D91 9395928C 83797069 66686A6C 6B686664 666C7581 8C949693 8D837971"
		$"6B686767 68686868 686A6D73 7A82898E 908F8C87 84828182 8282807C 78747374"
		$"777D8286 89898886 84828283 85898B8B 8A878584 84848585 84807C78 75737375"
		$"777A7D80 83858789 8A8A8A88 857F7A74 706C6B6C 6F73787C 8084888C 90949799"
		$"97938D87 82807F80 8181807E 7C7C7D7F 83888C8E 8E8C8884 807D7B7B 7C7E8082"
		$"83838281 8284878A 8C8C8A86 83818185 898E9296 98989897 9490897F 72625141"
		$"3632343C 4650575C 61656B73 797E807E 7A76716B 66626163 69727B83 888B8E91"
		$"979FA8B0 B3B2ADA3 9B938F8F 92979998 958F8A86 85878A8E 91929290 8D8B8A8A"
		$"8C8F9294 938F877C 726A6563 64666868 67656569 707B8690 95979389 7F746C67"
		$"6566686A 6A6A6A6A 6D71777F 878E9293 918B8783 81818182 817E7A75 7373757A"
		$"7F858889 89878584 84848688 898A8A88 87858585 84848280 7D797775 7576787A"
		$"7D808385 888A8A8A 8987837E 79736F6D 6C6C6E70 7274787D 83898F93 95969694"
		$"918D8985 83818181 807E7D7D 7E808387 8B8E8F8E 8C888581 7F7E7E7E 7E7E7F7F"
		$"80808183 84868787 87858381 8183868B 9095999D 9E9C9892 897E705E 4C3B312D"
		$"2F39424B 52565C62 6A747B81 827F7B75 706A6561 60636971 7981868A 8E9299A3"
		$"ACB5B9B7 B0A49A91 8E90959B 9D9B968E 87838387 8B909393 928F8C8A 898A8C90"
		$"93969591 887C7066 605E5F63 65656462 62666E7A 87939A9C 988E8478 6E676464"
		$"64666767 68686A6E 747D868F 95979690 8B858282 8283827F 7B75716F 71757B81"
		$"868A8B8A 88858484 86888B8D 8D8B8987 85858484 83817E7A 77757575 76787B7F"
		$"8286898B 8D8D8C8A 86827D77 736F6D6B 6B6B6D70 74787E84 898F9396 97969490"
		$"8C888482 807F7E7D 7C7C7C7D 8084888D 91939290 8C888583 81818182 83838383"
		$"83838486 888A8B89 8783807F 81868C92 989C9E9E 9D99938B 7D6B5741 3026242A"
		$"333F484E 5459616C 76808585 837E7870 69615C5A 5E66707A 82888C8E 939CA7B4"
		$"BCC0BCB1 A5978F8E 91989D9F 9C948D85 8282858B 8F929393 918D8B89 8A8C9094"
		$"9798948A 7E70655D 5A5D6064 64626060 656F7B89 959D9F99 8F817468 615D5D60"
		$"63666868 696A6E76 808C969D 9F9B948A 84808082 8383807A 75717072 777F858B"
		$"8D8D8B87 85838486 888B8D8D 8C8A8886 8482817F 7D7A7775 73727274 767A7E82"
		$"878B8D8E 8E8C8985 7F78726D 6A68696B 6D6F7377 7D838A90 94969795 928E8A87"
		$"84828181 81818080 80808185 898D9092 918F8B87 837F7C7C 7D7F8284 85858482"
		$"82848688 8A8A8885 83838589 8F969CA0 A09E988E 806C5741 31292932 3B434848"
		$"4A4E5766 75838A8A 867F7870 69635F5F 62687078 7F838688 8E98A4B2 BCC1BFB5"
		$"A99B9492 959DA2A4 A1978D84 80808389 8F939696 94908D8B 8A8A8D91 9597958D"
		$"8272655B 575A5F65 67676563 656B7583 8F999C99 9185796D 645E5C5E 6165696B"
		$"6D6D6E72 79838D97 9B9B978F 89838081 8283827E 7A757272 74787E84 888A8B89"
		$"88868585 87898C8E 8E8D8B89 86827F7B 79777575 75757575 76787B7F 83888C8E"
		$"8F8D8984 7E76706B 69696A6D 70737679 7D82888E 94999B9A 97928D87 84828284"
		$"85868582 7F7D7D7F 82888C90 908E8A84 807C7B7B 7D7F8284 84848381 81818488"
		$"8B8D8C8A 8785868A 91999FA3 A3A19B91 826F5A44 342A2932 3C464C4E 4F515865"
		$"73818A8C 89817971 69635F5F 62686F77 7D818486 8A929DAB B4BAB9B2 A89B9390"
		$"929AA0A5 A49C9389 82808186 8B8F9294 9492908E 8C8B8C8F 92969692 88796B5D"
		$"5554575F 64686866 676B737F 8A969C9C 968B7F71 665E5B5C 60666C70 72727273"
		$"78808A94 9B9D9991 89817E7E 80838380 7D797674 75797D81 85898A8A 89878686"
		$"87898B8D 8F8F8F8E 8B87827C 78757474 74757676 7777797D 81878B8E 8F8D8983"
		$"7D756E68 6565666A 6D707377 7C82888F 95999B9B 9995908B 87858486 888A8A88"
		$"85818080 83888C8E 8F8D8A86 817D7A7A 7B7E8185 86858380 7E7E8085 8A8E8F8D"
		$"8B89898D 929A9FA1 9F978973 5B413028 2B394652 5754504C 505C6C7E 898E8D87"
		$"7E736A62 5E606670 78808381 807E818B 98AAB6BD BBB1A496 8E8D929C A5ABAAA2"
		$"978A817D 7E848B93 97999793 8F8B8989 8B8F9294 928C8172 64585355 5B63686A"
		$"69656468 707E8A95 9A989084 786C645F 5D5F6268 6C707272 72747880 88909596"
		$"938D8781 7F808284 84817E7A 77767779 7D818486 87878685 85858789 8B8D8F8F"
		$"8E8C8A86 827D7975 73737476 77787979 7B7E8286 8A8C8C8A 86807A73 6D686666"
		$"676B6F73 777A7D81 868C9195 97979693 8F8A8785 85878A8D 8E8D8A85 82818387"
		$"8A8D8E8C 8985817F 7D7D7D7F 81848687 8683817F 8185898F 9292908E 8E909296"
		$"97958E80 6C543F2E 29303C4D 585E5E58 56585F6D 78818584 807A726A 64606268"
		$"717C8488 88838181 8896A4B2 B7B5AC9E 938D8D95 9DA6AAA8 A2988E86 8282858B"
		$"91979A9A 97928E8A 898B8E92 9290887A 6C5D5553 5660676D 6E6C6A69 6D77828E"
		$"94969288 7C6E635D 5B5D6268 6E727575 76767A80 878F9395 928C8680 7D7E8185"
		$"8787847F 7B797879 7C808386 87878684 83838486 898D8F90 908E8B87 827D7975"
		$"73737476 77797A7A 7C7E8286 8A8D8D8A 857D7670 6D6B6A6A 6A6C6E72 767C8187"
		$"8B8D8F91 92949492 908C8886 8587898D 8F8F8D89 86838283 85878888 87858482"
		$"817F7F7F 8185898C 8C8A8682 81858A92 96989793 908E8D8D 887E6E58 45352F33"
		$"3D4B545A 5C5A5B5F 656F767B 7D7B7876 726E6A66 666A717B 82888987 85848993"
		$"9EAAB0B0 AB9F968E 8D9299A1 A4A4A09A 938D8886 86888D93 989C9C9A 958F8A87"
		$"888C8F91 8B7E6F5D 514C4F59 636D7171 6F6B6D73 7B868E93 928B8173 665B5656"
		$"5A646D76 7B7B7976 777B838D 94989690 88807B7A 7C818588 8886837F 7C7A7A7C"
		$"7F838688 89878482 81818488 8D919292 8F8A847C 77737272 74777979 7979797B"
		$"7E83888C 8D8B867E 76706C6A 6A6C6D6F 7173767B 81878C90 92929292 9292908E"
		$"8B878686 878A8D8F 8F8D8B87 85848484 84858585 84848381 81818285 898D9090"
		$"8E8B8A8A 8C909496 96928D87 7E746757 493C383C 434F575C 5D5B5B5E 646E767C"
		$"7E7C7874 6F6A6767 6A70777F 84868785 84868B93 9CA5AAAB A8A09993 9092969C"
		$"A0A19E97 90898584 868A8F94 989A9B9B 9995918D 8C8C8D8D 887E715F 534B4B53"
		$"5D697073 74727377 7D858B8F 8E898074 685D5654 575F6973 7A7D7D7B 7A7B8088"
		$"8F959793 8D857F7B 7B7E8185 87878582 7F7B7878 7A7E8286 88898987 86848587"
		$"8A8E9092 908B847A 736D6B6E 73797C7E 7E7C7B7B 7E82878B 8B88837B 746D6967"
		$"686A6E72 767A7E83 888C9092 93939291 90908E8C 89868585 868A8D8F 8F8D8A87"
		$"85838282 81818182 83838383 8383858A 8F939595 93908F8F 90939494 90887D6F"
		$"5F4D4037 373F4854 5B5F5F5C 5C5F6670 777C7E7D 7A76716C 69676A70 79838A8C"
		$"8B878484 88929BA5 AAAAA69E 97919092 969DA1A3 A0999187 817E8086 8E969B9D"
		$"9D9A9793 908F8F91 918F887C 6E5D5149 49515A66 6E727473 75797E86 8B8D8C87"
		$"7F73695F 59565860 69737A7F 817F7E7E 80868C92 9493908A 85817F81 8386888A"
		$"8987837D 7977777B 7F85888A 8A898887 888A8C8F 9193918D 857B726A 686A6E74"
		$"797D7E7D 7D7D7F83 86888886 817B746E 68646365 696F767E 84888C8F 91929393"
		$"94949391 8D898581 7F808389 8D8F8F8D 8A868382 82848585 85848383 83838485"
		$"888C9095 98989693 908E8D8D 8B867B69 553F312B 303E4B59 6163625D 5C60666F"
		$"767A7B79 7773706C 6B6D727A 838D9293 918B8684 878F99A3 A9A9A59B 94909096"
		$"9CA4A7A7 A2988F85 7F7C7E84 8B959CA0 A09C9892 8E8E8F92 92908676 65534743"
		$"46525D67 6E707273 777D8389 8C8C8880 766A6159 55555A64 6D777E82 83818080"
		$"8286898D 8E8C8985 83818284 87898B8B 8A87837D 78757578 7D838789 89888888"
		$"898C8F92 93938F87 7E736B65 65696E74 797B7D7E 7F808284 86878682 7E78736D"
		$"6967676B 70777E85 8B909496 96969594 94949391 8E88837F 7E82878E 9395938D"
		$"87838182 83858585 84828284 86888A8D 9195989B 9C9A9690 8982796E 5E4A3828"
		$"252D3A4E 5C666965 63656A72 777B7A76 716D6A68 67696E76 7F8A9297 9896928E"
		$"8C8E9298 9DA1A09C 97918F8F 939CA4AA ABA79E90 857C7A7E 858F979D A1A19E9A"
		$"96929090 91938E82 715A483C 3B455364 6F737471 72767E88 8E908D85 7B6E6258"
		$"5250545E 69757F86 89878583 84888B8D 8E8C8985 817F7F81 848A8E92 92908A82"
		$"7B757375 7A818688 89878685 878B9096 999B9890 84766A61 5F636870 75787A7A"
		$"7B7D7F83 85868581 7D77726C 67646569 6F777F88 8F939799 9A999896 94949391"
		$"8E89847F 7E81878F 95989792 8D878383 83838382 82828385 888B8E90 94989C9E"
		$"9C968E82 74645240 3126252D 394A5862 66646468 6E777D80 7E766F69 65636468"
		$"6F77828E 979B9B97 95939498 9B9D9D9B 9793908F 9093979B A0A4A5A3 9D938A80"
		$"7C7E848E 969C9F9F 9E9C9997 9593918E 877B6C58 473A3840 4D5F6B73 76757679"
		$"7F878B8D 89817669 5E544E4C 515B6674 7F878B8B 8A888789 8A8C8C8A 88848280"
		$"80818488 8E949796 91877E75 72747981 878B8B89 8785878B 8F95999B 99918677"
		$"6A5F5B5D 636C7378 7B7B7B7D 7F818181 7F7B7773 6F6B6765 6569707A 858F979B"
		$"9D9D9C9A 97959392 91918F8C 89858587 8C949A9E 9D99938C 87848383 84848586"
		$"888B8E92 95979898 95908678 6347311F 191D293C 4B575E60 62656B75 7C828380"
		$"7B736C64 605E6169 7585929C A1A19F99 97979AA0 A4A6A49C 948B8787 8D979FA6"
		$"A8A49E94 8B837F7F 838B929A 9FA1A19E 9B979696 9799958C 7C644F3D 34353E4E"
		$"5C686F71 74777C83 888B8B87 7F74685A 5049484D 57667480 888C8D8B 8A8A8C8E"
		$"90908E8A 86817E7E 80848A90 95979690 887E7875 767B8187 8B8C8B89 88888B8F"
		$"94989996 8F837668 605C5D63 6A707578 7A7A7B7B 7C7C7B7B 7A787571 6D696768"
		$"6D757F89 92989C9E 9E9C9995 93939394 9494928E 8B8A8D93 999FA19F 9A928C86"
		$"84848485 8687898B 8E929494 918B8173 5F46301C 1418253A 4B575D5B 5C5F6877"
		$"828A8A84 7C746D67 6463666D 7887949E A4A4A19B 9A9C9FA5 A9ABA8A2 9A908A87"
		$"8A929BA6 ACADA698 8A7C7678 7F8D98A0 A4A3A09C 98959496 989A978F 80685039"
		$"2E2F394B 5A666C6C 6D707680 86898882 7A6F6459 5049484C 56657482 8A8E8E8C"
		$"8A8A8B8D 8F91918F 8D898581 81848A92 999D9B93 887C7472 757D848B 8F8F8D8B"
		$"8A8B8D91 95999997 8F837566 5D595A61 686E7272 72717171 73767A7E 8081807C"
		$"78747170 7379818A 92989B9B 9A969390 90929599 9A989590 8D8D8F95 999D9D99"
		$"948C8784 83838485 86888989 87817666 523A291D 1B243040 4B535654 565A626E"
		$"78808484 817C7772 6F6F7279 838F99A0 A3A2A09C 9A9CA0A6 AAADACA6 9F979290"
		$"9094989D A0A19E97 8E827A77 7B85909C A4A7A59F 99939090 90918E86 7A685747"
		$"3E3C424E 5963696D 70727477 79797875 70686057 504B4B51 5A687581 898D8D8B"
		$"89888889 8B8D8F90 908E8C8A 89898C91 95999892 897D7673 757D858D 9191908D"
		$"8B8B8C8F 92959592 8C82776B 625C5B5D 62686D71 7577797A 7B7D7F82 84858482"
		$"7E7A7674 74777D85 8D969DA1 A19F9B96 9494969A 9C9D9C98 94919194 979B9C9C"
		$"9995918D 88847F79 736B5F50 3F2B1C14 16233346 535B5D59 595D636C 72767674"
		$"72717274 767A8089 94A0A9AF B0ACA69F 9C9C9FA5 AAAEAEAC A79F9994 93979BA1"
		$"A29E9688 7D75747A 84909BA3 A8AAA9A5 9F97918B 8886827A 6E5C4C3E 3B414D5F"
		$"6C757979 797A7B7C 7B777169 625C5650 4C4A4C52 5C6A7783 8A8D8E8C 8B898787"
		$"888B8E92 9392908C 8B8D9096 98989287 7D736E70 76808991 96989896 9492908E"
		$"8D8C8985 7D71665A 5351545C 65707A82 898D8E8C 8A888684 8383817F 7C797777"
		$"7A808791 99A1A6A8 A6A29D97 93939498 9A9B9B99 97969799 9B9D9C9A 958F867B"
		$"6A523616 0501040D 203E525C 5F595656 5C667078 7A78736D 67636264 6B798AA0"
		$"B1BDC1BE B8AEA7A3 A3A9AFB5 B7B5B0A8 A0989597 9BA2A6A6 A1958777 6C676A75"
		$"8496A5AF B3B0AAA0 97908C8A 86807462 4F3D3332 39475666 727C848A 8D8D8A83"
		$"7A70665D 554D4844 4448505D 6A788289 8D8D8C8A 88868688 8B8F9295 96969595"
		$"96999B9D 9A92887A 716B6C72 7B879096 9A9B9B99 9795928E 8880766B 6157514F"
		$"50555D69 7684909A 9F9D9991 8B858282 82828180 7F7F8082 868C949C A4AAADAB"
		$"A59B938C 8A8D9299 9EA0A2A2 A19F9D9A 948A7A66 4A260F05 0000081A 304A595F"
		$"6161646C 72777773 6D676361 6161656B 798DA1B5 C3CAC9C1 B8AEA9A9 ABB1B5B8"
		$"B8B4AFA7 A29E9FA3 A8AEACA2 927C6D63 636D7A8A 97A3ABAF AFADA79D 9389827C"
		$"72655440 31282B39 4A5E6D78 81878D93 94928A7D 6F5F534B 45414144 4B556270"
		$"7C858A8C 8B898682 807E8085 8B919496 9695979B A1A7A6A0 9482746A 686E7884"
		$"8E949797 97959189 827C797A 7B7B766C 61565357 6171808E 989EA0A0 9D97918B"
		$"8785868A 8C8D8D8B 89898B91 979FA5A9 ABA9A49E 98928F91 949A9D9F 9C968975"
		$"59371C09 00000715 24343B39 38384155 697F8A8A 8375695F 5856575C 67798EA7"
		$"BAC6C9C3 BBB2B0B4 BCC6CDCF CBC1B5A8 9F9B9CA3 ABB3B5B2 A898897B 74747983"
		$"8E99A1A6 A6A29B93 8C878380 786C5B45 3529262E 39485662 6E7C8997 9E9F998C"
		$"7E6E635B 544F4A46 464B5462 6F7B8488 8A8A8A8A 89878686 86878889 8B8D9094"
		$"999E9F9C 95897F76 7375797F 8487898B 8D8F9192 92908D89 8785817C 766E6762"
		$"61656C78 84919A9F A09D9993 8E8B8A8A 8B8B8A8A 8988898B 8F959DA7 AFB5B7B5"
		$"B0A8A097 8E867967 5137200C 04060F20 2F3B4345 4646484C 5155595C 5F63666A"
		$"6C6C6D70 7884919F AAB0B1AE ABA7A8AC B2BAC2C8 CACAC6C0 B9B3AFAD ABA9A6A0"
		$"978C837B 7A7E8590 9BA5ABAF AEAAA296 8A7C7167 5D52473C 3531333B 45515C66"
		$"6F778088 8C8C8880 766B635D 5A5A5C60 656B7176 7A7C7D7D 7E7F8080 81838589"
		$"8D919597 97979798 9898938A 7E6E625C 5D677280 8A909598 9CA0A3A3 A19D9893"
		$"8F8D8983 7D777371 73798088 8E929494 928E8984 807C7B7D 80868B8F 92939495"
		$"989CA1A7 AAAAA59D 8F7C6344 280F0202 0E263D51 5A585248 4549505A 6165645E"
		$"59535050 55617083 95A5B0B4 B2ACA59F 9DA0A7B2 BDC7CBCA C5BBB3AC AAADB1B6"
		$"B6B1A89A 8F868385 8A939CA5 ACB1B3B1 AA9E9183 7870675E 513F3125 22293545"
		$"51596064 6A747D85 87847E74 6B645F5C 5957585C 65717D87 8D8D8C88 85838486"
		$"8A8E9296 98989591 8D898888 86847F77 716B696C 737D8894 A0AAB1B5 B5B1AAA0"
		$"99938F8E 8D8B8781 7A746F6D 6E737A84 8C939694 9088817C 7B7D8084 86878888"
		$"8A8E9296 9A9C9D9B 917F623C 1E0A0207 1A3B5464 69635F5B 5D666D73 74706B64"
		$"5E585452 555E6E84 9AB0BCBF BBAFA49A 9699A0AA B3BBBDBB B5ADA6A1 A1A5A8AC"
		$"AAA49B8F 86828389 919BA5AF B6BAB9B5 ACA09488 7C706151 402F2626 2D3B4854"
		$"5C61676D 757E8384 80776C60 554B4644 48525E6E 7B878E90 908E8C8A 8A8A8C8E"
		$"92969999 96908A84 84888D93 938E8578 6E686973 7F8F9CA7 AEB2B3B2 AFA9A39B"
		$"94908C88 847F7971 6C68696E 76818B94 999A9892 8D898684 84848688 8B8E9193"
		$"9290887B 674B2E11 0200091B 2F475559 5B5B606C 78868C8A 84786E64 5E5A585A"
		$"606C7B8E A0B0B8BA B7B1ABA6 A5A7AAB0 B3B4B1AB A49C999B 9EA3A5A3 9D938A83"
		$"8183878D 949CA5B0 B9BFC0BA B1A39587 7767543F 2E201D23 2D3C4850 585E6874"
		$"808E9595 8D7D6D5B 4C403A3A 404C5B6D 7C899092 908C8885 85878A8E 9091908D"
		$"8B8A8D93 9BA3A8A9 A59B9186 7F7B7B81 89939DA8 B1B8BBBB B6AEA499 9089837D"
		$"78726C66 63616369 727F8A94 9A9C9A96 918C8886 85868789 857D6745 270D0000"
		$"06132C51 666A665B 585C687B 8A949895 90888076 6E66656B 77899BAB B2B0A99F"
		$"99989EAA B8C8D2D6 D1C3B29F 94909197 9B9C9890 89838185 8A92999F A6B0B8BE"
		$"BEB8AD9F 9286796B 58422E1C 161B2636 424B5154 5B657180 8B918F86 79675646"
		$"3C383A43 50607181 8C949692 8B837E7D 818A949F A6A8A59E 9A989BA3 A9ADAAA0"
		$"94867D7A 7C838B94 9DA5ACB3 B7B7B3A9 9E928882 7F7F7D7A 746C6662 63697482"
		$"8F999FA0 9F9B9894 8F8B8377 654B2E0F 00000000 0D273E54 60636668 6E788189"
		$"8E8E8D89 86848280 81838892 9DA9B1B4 B1A79D93 8F9199A8 B8CAD4D6 D0C1B2A4"
		$"9B979595 91898078 7579808A 93999FA7 B1BDC5C9 C6BAA995 82705D4B 39271B17"
		$"1B273442 4C535A62 6C78838B 8E8A8174 66584C42 3E3E444E 59656F77 7E828484"
		$"8383868C 96A6B4C0 C4C1BBB1 AAA7A6A8 A59D9181 756D6C72 7A848F99 A4B0B9C0"
		$"C1BCB2A2 93847A75 7272706D 69646365 6C798899 A6AFB3B3 B0AAA298 8872532B"
		$"11050000 00010F2A 3C444B52 5E6F7F8D 95979693 92949492 8F8A8889 8F99A2AA"
		$"ABA59E94 8E8C8E95 A0AEBBC9 D1D3CFC5 BBAFA69E 948A8078 74747982 8B939BA1"
		$"AAB6C0C9 CCCAC1B3 A2907B63 4A301B0D 09111D2D 3A42494D 535C6773 7B7F7D76"
		$"6D61564C 4745464C 545E666C 71747678 7A7C8189 95A5B5C7 D3DBDDD9 D3CBC5BF"
		$"B7AFA599 8F878383 868A8F93 979DA4AC B1B5B3AB A0928579 716F6E70 70706F6E"
		$"7074797F 83858075 634B2F0F 00000000 050F1F35 43494D4F 545D6874 7D828482"
		$"80808287 8F9AA8BA CADAE3E5 DFD1C1AF A1999699 9FA9B0B4 B5B1ACA8 A6A8AAAC"
		$"A89F9488 807C7D84 8B939AA0 A8B1B8BD BBB1A492 81706050 3F2D2119 1A263446"
		$"52595C5A 5B5D626A 6D6B6458 4C403B3B 3F48535F 6A757E85 8A8C8C88 837F7F83"
		$"8C9BACBE CAD1D3D0 CCC8C4C0 B9AFA395 8C87878B 8F95999B 9EA2A7AD AFAEA79B"
		$"8E807771 7072767B 7F838688 898A8986 7C6B5437 1E0A0000 0000040E 1D31424E"
		$"59626A72 787C8082 82807E7C 7A7A7E86 93A3B3C2 CCD2D2CC C3B8AFA8 A5A5A7AD"
		$"B1B4B4B1 ADA8A5A4 A3A19C94 8C848181 848B939C A5AEB7C0 C6C8C4BA AB99846C"
		$"533B2717 11131E30 4355626A 70747779 78746E64 57493C31 2A26272D 37455567"
		$"76838C90 918F8D8D 9097A1AD B7C1C7C9 C7C3BEB9 B5B1AAA0 978D8785 868C929A"
		$"9FA3A6A8 AAACACAA A49A9084 7B736E6E 70747A81 89929797 8E7A5F3D 210B0000"
		$"00000510 1A242E38 45556474 808A8F90 8F8D8B89 89898D95 9FACB8C3 CACDCAC0"
		$"B6ACA5A1 A0A0A0A2 A3A4A5A7 A7A6A5A3 A19E9C9A 9896938F 8E8E9197 9EA6AEB4"
		$"B7B7B3AC A1927E66 4E362519 14161E2A 36424D57 606A727A 7D7D786F 655B5149"
		$"43404042 47515D6B 7A88929A 9D9D9D9D A0A6ACB4 BBC1C6CA CAC8C3BB B2A8A19B"
		$"97959595 9696989A 9C9EA0A2 A3A3A19D 98908981 7B767270 6C675E52 4333251B"
		$"130D0907 080E1725 313D454B 50545A60 676D7377 7A7D8288 8F98A1AA B2BAC0C6"
		$"CBCFD0D0 CDC9C3BC B5AEA9A5 A3A3A2A0 9D999490 8E8E8E8F 9193979B 9FA3A4A4"
		$"A29E9D9D 9E9E9B95 89796754 44362C24 201E212B 38495865 6E727577 797C7D7C"
		$"78716A63 5F5F6268 6E767D83 878B8D8F 8F8D8C8A 8A8C8E90 93989DA1 A4A4A19D"
		$"98938F8B 88868585 868A8E94 989C9EA0 A1A1A1A1 9F9B9793 908E8F91 93938B7B"
		$"674E3620 0F050000 040C1B32 44525B5F 656B7480 8A929696 938F8B89 898B8E94"
		$"9AA2AAB2 B9BDBDB9 B5B1AFAD AAA7A4A0 9C989591 8D898684 84858687 898D9298"
		$"9EA5A9A9 A8A5A4A4 A6A8A8A4 9988745E 48342315 0E0C121F 2F425260 6A72797F"
		$"84888988 847C746B 635D5A5B 5F666E78 81899197 9C9E9F9F 9FA1A3A5 A8ACAFB1"
		$"B2B0ACA7 A29E9A96 918D8B8B 8D9299A2 A8ABACAA A7A5A29E 9B999590 87796449"
		$"2D0F0000 00000000 0207162F 45576267 6B6D7177 7D838585 837F7F81 868F99A5"
		$"B0BAC4CE D5D9D9D7 D4D0CCC8 C6C5C4C2 BFBBB4AC A2968B83 7D7A7876 76777C85"
		$"8F9AA2A7 AAAAAAAB ABA9A295 826A523A 291E1715 1518212F 41576979 83878784"
		$"817E7A74 6D645C54 4F4D5056 5C646A6E 7379808A 92999EA0 A09FA0A3 A7ADB0B1"
		$"AFABA6A1 9D9B9896 94929293 98A0A7AE B2B4B4B3 B4B6B8BA B8B2A89A 866B4D2C"
		$"15070000 00000000 0A1E344C 5E696F71 7273767A 7C7D7C7A 797B7F85 8B92989C"
		$"A3ABB4BE C3C4C2BE BAB8B8BB BDBDBBB7 B1A9A096 8D857E78 74747579 7E858F9B"
		$"A6AFB5B8 BBBDBEC0 BEB8AC99 836A5646 3A322C28 272A3341 50606B71 7472706E"
		$"6A645C53 4B454243 464C525A 61697076 7C848A8E 93999FA5 A9ADB0B4 B7BABCBC"
		$"BAB7B3AF ACAAA9A9 A8A7A7A9 ACB1B5B9 BAB8B6B2 AEA9A197 8670573B 23110602"
		$"00000000 00000D29 41566367 6A6C6E70 72737372 7272767D 869099A1 AAB3BBC3"
		$"C9CED1D1 CFCBC7C3 BFBDBBBA B8B4AEA6 9D958F8B 89898886 85868A92 9AA2AAB0"
		$"B3B5B5B5 B1A99C8A 78685B52 4B463F35 2C242226 2E3A4754 5D636563 605C5958"
		$"57565450 4F4F5157 5D656C74 7C848D97 9FA5AAAC ADAEB1B5 B9BDBFBD BAB6B2B0"
		$"AFB1B2B2 B1AFAFAF AFAFADA9 A6A4A5A9 ACAEA99E 8B705539 220E0301 00000000"
		$"00000A1E 3244525A 6168717B 82888A88 85828284 8A929AA2 AAB3BDC7 CFD5D8D8"
		$"D5CFCAC5 C1BFBDBB B7B1AAA2 9B95908D 8A888684 868A9099 A1A8ACAE B0B4B6B7"
		$"B2A89985 725E4B3B 2D211812 0E0C101A 28384857 636B6F71 72726F69 625B5652"
		$"535A636D 767E8386 8B9199A4 ADB3B7B8 B9BABCBF C1C3C1BD B7B1ADAB ABABABAB"
		$"AAA9A9AB ADB1B3B3 B1ADA9A3 9E988E7F 6B52381C 0B030000 00000000 0103122D"
		$"43535F67 70787F87 8B8D8D8D 8D8D8E90 939599A1 AAB4BFCB D4DADEDF DFDDD9D5"
		$"D1CDC9C6 C1BBB2A7 9C928A84 7F7A7777 7A828B96 9FA5A7A7 A7A7A7A7 A39D9385"
		$"76665545 35251910 0D111A2A 3B4D5B66 6E72767A 7E81817F 7A726C66 62606062"
		$"64666B71 7880888E 9396999C A0A4A9AF B2B3B2AF ADACABAA A9A7A6A4 A5A7ABAF"
		$"B3B8BBBC BAB5AC9E 8B72573B 220C0100 00000000 02061327 39475561 6D7B8791"
		$"989B9D9D 9E9E9E9E 9D9C9D9F A3A8ACB0 B2B3B4B5 B6B8B9B9 B9B9BBBD BDBBB4AA"
		$"9F948B83 7F7D7D7D 7E828890 989FA4A8 ABADACA9 A3998B7A 6957473A 312B2828"
		$"2B323B47 525C646B 70737678 7978756F 69646160 61636668 696A6D72 787E8488"
		$"8B8E9296 9CA2A8AC B1B5B7B8 B8B6B3AF ABA8A7A7 AAAEB3BA C1C7CBCD CBC7BDAF"
		$"997C5D3C 210B0000 00000000 040D1D35 46525D67 717C8387 8783817F 7F818386"
		$"898B8F95 9EAAB4BC C0C0BFBE BFC1C4C6 C8CACBCB C9C5BFB5 ABA1978F 88827E7A"
		$"7A7D848E 98A2ACB4 BAC0C2C0 B9AD9C87 725D4A39 2A1E1510 11192433 414E5A64"
		$"6B717475 74706C68 65656668 69696968 686A6D71 7477797B 7D81868C 9298A0A8"
		$"AFB6BBBE BFBFBDB9 B4AEAAAA ADB3BAC0 C5C9CAC8 C2B7A997 8065492D 17070000"
		$"00000000 06121F2B 3844515D 6A778189 8E90918F 8C8A8A8C 919AA4AE B7BDC0C0"
		$"BEB9B6B4 B4B4B5B5 B6B7B8B8 B7B3AFAB A6A09B95 8F8A8785 868A8F97 9FA7AEB2"
		$"B3B1AAA0 94867769 5B4E4238 2F292627 2C343D47 50596065 68686969 69686766"
		$"676B6F73 74747372 72737577 78787879 7C818890 99A3ABB3 B9BDC0C0 BFBDBDBD"
		$"BFC1C4C6 C7C7C5C1 BBB3A694 816C5640 270D0000 00000208 162C3C46 4D515861"
		$"6B757B7F 807E7E80 81828282 83878E9A A6B2B9BD BDBBB9B7 B5B1ADA9 A8AAADB2"
		$"B6BABBB9 B7B4AFA9 A096908E 9199A0A7 ABACADAE AEAEAAA4 998A7B6B 5E53493F"
		$"33261B13 12182434 424E565C 60626363 6362615F 5E606368 6D727679 7C7E8185"
		$"888A8B8B 8D90959B A2AAB2B8 BDC1C6CA CDCFCECC CAC8C5C2 C1C1BFBC B5A99B89"
		$"77655038 210B0000 00000717 27373F41 43464C55 5D63686A 6C707478 7A7A7874"
		$"74798391 9EAAB2B8 BCBEBEBD BCBABABA BABABBBB BCBCBCBD BEBEBBB6 B1ABA8AA"
		$"ADB3B7B9 BABABABA B8B4ADA3 9687796B 5D4F3E2B 19080000 05111E2E 3A444B51"
		$"575D5F5F 5D595654 55585C62 676B7074 7A828A92 97999B9D 9FA1A3A6 A9ADB1B7"
		$"BDC3C7CA CDCFD2D4 D5D4D2CE CAC6C2BE B7AD9C84 694B2E12 03010000 0000091C"
		$"2B353B3F 42454B53 5C656C70 73747575 74716F6F 727A838F 99A2A9AD B2B8BDC2"
		$"C7CBCFD4 D9DEE1E1 DED9D2C9 C0B6ADA3 9B959395 999FA3A5 A6A7A7A5 A29C9488"
		$"79675440 2B170903 00000511 223A4F63 737F8688 89898785 817D7874 706D6B6A"
		$"6C707478 7C808488 8B8D9093 969A9EA2 A7AEB7C1 C9CFD1D1 CFCBC6C2 BDB8B2AA"
		$"A39F9B97 8F816C52 35170602 00000000 030A1117 1D232C38 46586878 85909AA4"
		$"AAACAAA2 9A928E8E 929AA2AA AEAEB1B5 BAC1C7CB CDCECFD0 D2D6D9DD DDDBD7D1"
		$"CAC4BCB4 ACA59D95 8D85807E 7C7A756F 675F574F 463A2B1B 0E040000 03091424"
		$"33414D58 61686F75 7C848A8F 91908F8D 8D8D8F91 9395989D A2A8AAAA A8A5A3A3"
		$"A5ABB0B5 BABEC3C7 CCD2D4D2 CFCBC7C4 C0BAB3AD A49A876C 491E0602 00000000"
		$"00000612 1F2B3742 4D58636E 777E848A 8F939491 8C86817D 7E848F9D ACBAC5CB"
		$"D2DAE0E4 E4E2DED9 D2C8C1BD BBBDBEBE BDB9B2A8 A0989597 9A9FA3A6 AAAEB2B5"
		$"B4AEA394 826D563C 230B0000 00000000 091D344E 6374818B 939A9D9D 9A948C83"
		$"7A726C68 6666676A 727E8C9B A5ABAEAE AEAFB2B6 BBBFC1BF BEBCBDBF C0C0BDB7"
		$"B1A9A4A0 9A94856D 4D250C04 00000000 07162636 41474A4A 4947484C 535D6975"
		$"8393A1AB AFADABA7 A6A8ABAF B2B4B3B0 AFAFB2B6 BABCBFC1 C5CBD0D6 D7D4CDC1"
		$"B5AAA29C 99979797 999B9EA2 A5A7A6A3 9C918372 604C361E 0D040000 00000716"
		$"283C4F5F 6E7C868D 90908E8B 89898A8C 8D8C8B89 898A8C8F 93979896 94908E8E"
		$"8F92979D A4AEB8C3 CCD2D3D0 CBC5BFB9 B4AFABA7 A29C9182 6C4E2F0F 00000000"
		$"0000040E 161D2327 2F394759 6A7A8894 A0ABB1B1 AEA69E96 918F939D A9B7C0C6"
		$"C8C7C5C1 BEBCBABA B8B5B3B1 B2B4B6B7 B6B2ADA6 A2A1A3A7 A9AAAAA8 A8AAACAE"
		$"ADA89E8E 7D6B5A49 37231206 00000000 0A1E3246 56636F7A 848D9396 97969490"
		$"8C86827F 7B777472 757B8189 8F939595 9494979D A3ABB3BC C4CACECF CECBC6BE"
		$"B7B2AEAC A2927958 36120000 00000000 0C243848 52565756 56585A5C 61697381"
		$"8D979EA2 A4A5A5A3 A4A8ADB4 B9BCBFC1 C0BDB7AE A9A7A6A6 A6A6A7AB AEB0B1B1"
		$"AEA8A19A 989CA2AC B3B8BBBD BDBDBAB4 AA9C8C79 634A3117 07020000 0000081A"
		$"304B6276 848E9498 99989795 9494918C 88848180 80808183 8587898B 8D8F8E8B"
		$"8A8B9099 A4B0BCC6 CCCECDC9 C4BCB6B2 ADA79D8F 76512F0F 00000000 02071732"
		$"454F575B 5D5F5E5A 59595E66 738595A2 A9A9A7A1 9FA0A3A8 ADB3B6B6 B7BABEC2"
		$"C3BFBAB2 AAA4A2A4 AAB2B8BC BBB6B0A8 A29E9DA0 A4A8ACB0 B6BCBFBF BAB0A18F"
		$"7B675139 220B0000 00000000 0B233A52 64727E88 91999EA1 A2A09E9D 9B989490"
		$"8B868381 82888D92 93918D87 8483878F 99A5B0BB C3C7CACC CBC8C3BC B5AEA69E"
		$"8C704C20 07020000 00000C24 394B575B 5E5F5F5D 5B585656 5A646F7D 89939A9C"
		$"A0A4A7AB B0B6BBBF C3C5C6C6 C5C1BBB4 AEAAA7A5 A8B0B7BF C0BBB5AD A6A09D9D"
		$"A1A7AEB4 BBC1C5C9 C8C2B9AB 99846D54 381A0802 00000000 030B1D3A 53677885"
		$"8F95989A 99969493 9393918D 8A878584 85898D93 97999996 938F8D8C 8F959EAA"
		$"B5C1C9CD CFCECBC5 BEB8B1AA 9E8C6D41 210B0000 00000613 2743555D 62656767"
		$"635B544D 4C505B6D 7D8B9396 999CA2AB B4BDC3C5 C6C4C4C5 C5C3BEB7 B0A9A4A0"
		$"A2A8AFB8 BCBCB7AF A69E9896 979BA0A6 AEB7C1CB D1D3CFC5 B5A18A70 54381F0A"
		$"00000000 00000D28 435D717D 868C9195 96949391 90908F8E 8C898682 8081868F"
		$"979EA19F 9B95918D 8D9198A3 AFBDC6CA CCCDCDCB C6BEB7AF A69A815D 38120000"
		$"00000206 1A3E5765 6C6E6E6C 6760584E 49494F5D 6B79838B 91969DA5 AFB9C5D1"
		$"D8D9D7D2 CCC4BAB0 A8A3A0A0 9F9EA1A7 ABABA8A0 9B99999D A2A8ADB3 B7BBC0C7"
		$"CDD3D3CD C1AF9B85 6B4E3010 00000000 0000091B 34546D7F 8B919597 9796938E"
		$"8B898683 807C7876 75777D88 939DA4A6 A6A29C95 908E929C A7B3BDC4 C9CCCBC5"
		$"BCB1AAA6 A0988260 3B130000 0000030B 1F405868 747C8080 7B6F6151 47444957"
		$"65758088 9098A2AE BAC8D3DB DCD6CEC6 BDB4ABA3 9D999490 8D8B8F9A A4ACAEAC"
		$"A9A5A2A0 A1A5ABB3 BCC4CCD4 DADEDCD6 CAB8A38B 72583B1B 08020000 00000511"
		$"284C6676 8084888E 9193918B 87858484 84848381 80808590 9CA8AFB1 AEA79F95"
		$"8F8C909B A7B5BEC2 C2BEB9B3 B1B3B1AB 936B4115 00000000 00011438 54697C8C"
		$"9AA4A59F 92807268 63636363 625E6169 778B9FB3 C5D5DEE0 DED8CFC3 B7A99E94"
		$"8B837F7D 818A939D A2A4A4A2 A2A2A4A8 ABADB1B7 C2D0DCE5 E7E3D9C9 B7A28C74"
		$"57341A08 00000000 00001236 56748997 A1A9AFB3 B1A79D91 88827E7D 7A76716C"
		$"6E788698 A7B2B6B2 ACA29B96 979EA5AC B0B2B1AF ACA8A7A7 A39C8763 3C140000"
		$"00000000 133B5B73 8797A4AF B2AC9F8D 7C6C6059 54514D49 4B536582 A0BFD8EC"
		$"F7FAF8F0 E4D6C5B2 A08F8072 6A666B77 85939DA1 A4A6A9AF B5BBBDBC BCBCC2CD"
		$"D7E1E3DE D3C2AE98 80684C2C 15070000 00000000 0F2D4D6F 899CABB5 BABCB7AB"
		$"9E90847B 75716E6A 68686F7D 8FA7B8C3 C7C3BCB2 ACA8A8AA ABABABAB A9A7A39E"
		$"9B9B978F 78523010 00000000 0001153B 596F8395 A4B0B3AD A1908175 6C66615B"
		$"5551535D 6E869EB8 CCDCE5E7 E5E0D8CD C0B1A292 8477706E 7179828A 9197A0AA"
		$"B5BFC6CC CFCFD0D2 D6DCE0E2 DFD9CDBC A78F765C 3F200C04 00000000 00000D28"
		$"45647C8C 9AA6AFB5 B3ABA093 89817C79 75716E6E 758394A8 B5BBBBB3 ACA5A3A5"
		$"ABB3B7B7 B6B2ADA6 A2A09F9F 988C734D 2C0E0000 00000105 1A40607A 8E9CA8B0"
		$"B1ACA292 83756C68 635D5753 545B697D 94AEC3D3 DDDFDFDD D7CEC1B0 9F8E7F71"
		$"6A696E7A 858F969C A2AAB1B9 C0C6CACC CDCFD3DA DFE2E0D9 CDBBA68E 755B4022"
		$"0E040000 00000000 0C264468 8396A6B2 BABEBCB3 A89C9085 7D797571 70707680"
		$"8E9FACB4 B5B1ACA6 A5A9B0B8 BDBEBCB6 AEA49D99 948E7E66 43160000 00000000"
		$"0A1E3755 6F8599AC B9C1C0B6 AA9A8C80 75695D4F 47434856 6A849CB4 C5D0D9DF"
		$"E2E3DDD1 C1AE9A85 73656064 6D7B868E 96A0A9B3 BDC7CFD5 D8D9DDE4 EBF1F2EE"
		$"E4D5C1A8 8D705233 1B090000 00000000 00011438 5873899B A9B4BABA B7AFA599"
		$"8E867E78 726D6D72 7D8F9FAD B6B8B6B0 ADADB1BB C3C9CBC7 C0B4A99F 97908474"
		$"572F1406 00000000 030A2044 627A8FA1 B0BDC3C1 BAADA195 8A827666 55433731"
		$"384A6280 9DB9CFDF E9EFEDE5 D7C4B19D 8C807978 7B81888E 9295989B 9FA5ACB4"
		$"BBC1C9D1 DAE4EBF0 EFE8DBC8 AF91714F 2F0F0000 00000000 00000A1E 3858738B"
		$"A0B4C3CD D0CBC2B4 A89C928A 827A726C 6C727F91 A1AFB6B7 B7B7BAC0 C8D1D5D3"
		$"CCC0B4A8 A09A9185 6B43240C 00000000 00000E2A 435A7189 A1B9C8D0 CEC4B6A6"
		$"988C7E6D 5A46372E 313F5677 96B2CADE EDF8FBF6 EAD6BEA2 89756B6C 727E868C"
		$"8F8F9195 9FADBBC9 D2D5D8DA E1EBF2F6 F5EDE0CD B79D8062 411E0903 00000000"
		$"0000040E 2446637B 90A4B4C0 C8CAC4B7 A99B8F85 7B6F665F 5F65748A A0B5C2C6"
		$"C6C3C5CD D5DFE2DF D6C6B8AA A19D978F 764C2A0E 00000000 00000C25 3D536C88"
		$"A2BBCACE CCC2B8AE A49B8E7C 6851423C 3E48576B 7F93A6B7 C8D9E4E9 E8E0D1BC"
		$"A48A7464 5B595A60 67717D8B 9CB1C5D7 E4EBF1F5 F9FDFFFF FEFDF2DC C3A68869"
		$"48250F05 00000000 00000208 19365067 7E95A7B3 BABAB7AF A59B9086 7B6F6660"
		$"636D7C8F 9FACB3B3 B3B5BCC9 D6E2E7E5 DCCBB9A6 988F857B 6440230B 00000000"
		$"00000E2C 49657E94 A9BBC5C8 C5BBB2A9 A29D958B 7C68584C 474B5464 75899BAD"
		$"BED0DCE4 E2D8C7B0 987E6A5B 54555A62 686C737D 8A9CAFC3 D2DEE6EB F2FAFFFF"
		$"FFFFF5E1 C9AD9174 55351B09 00000000 00000000 0F2F4C68 8096A8B7 C1C5C3BD"
		$"B4A89E94 8C847D75 71727882 8D99A0A4 A7A9B0BC C7D3DADC D8CCBFAF A2968879"
		$"613F220B 00000000 00000E2B 48647C90 A4B8C7D0 D1C9C0B7 B1AFA89C 8B735F4F"
		$"484A5464 748493A2 B2C4D0D8 D9D3C8B8 A5917E6D 625C5B5F 656F7984 909CAABA"
		$"C8D6E2EC F5FBFFFF FFFFF9ED D9BD9F7E 5C381C09 00000000 00000000 0D29445F"
		$"778B9BA9 B3BBBDBA B4AAA198 908A847F 7E828994 9EA8ADAE AFAFB4BE C9D5DBDD"
		$"D7C9BAAA 9C8F7A5D 3B130000 00000000 01031435 536F8CAA C3D7E1E1 DCD2C8C0"
		$"B7AD9F8B 7763564F 4D515967 758494A6 B6C6CFD1 CDC3B4A0 8B75655C 595C6064"
		$"6B737E8E 9FB2C4D4 E0E8F1FA FFFFFFFF FFFFF2D9 BC9C7A58 35110000 00000000"
		$"00000208 1A38536B 8299ABB9 C1C3C1BB B2A69B91 877F7B7C 828E99A5 ADB1B4B4"
		$"B8C0CBD9 E2E8E5DB CCB8A491 7A5F3D14 00000000 00000000 0D274361 7F9BB5CD"
		$"DDE7E9E4 DCD0C6BE B2A28D73 5F4F4644 4C5C6C7C 8A96A4B5 C1C7C8C2 B7A7947F"
		$"6F656165 696E7274 797F8B9C AFC5D7E5 EFF7FCFE FFFFFFFF F7E8D1B1 906E4C28"
		$"11050000 00000000 0000091C 3656738F A6B8C4C8 C8C4BCB1 A79E968E 86807F84"
		$"8C96A1AC B4B9BEC4 CBD3DADE DCD6CCBE AF9F8D7B 5D331607 00000000 00000612"
		$"FFFFFFFF FFFFF7E7 D4BEA790 75563511 00000000 00000000 00000819 2E465B6B"
		$"767C8187 909DAAB6 BEC2C8CE D6E0E7EC EDEBE7E3 E1E2E5E9 E9E3D2B5 86441A08"
		$"00000000 00000000 00000E2B 4C7191AB C3D8EAF8 FFFFFFFF FAF0E0CA BBB1ACAB"
		$"AAA8A6A4 A19E988E 857B6F61 5343352B 27292C30 353B434D 5B6B7B8D 9DADBFD5"
		$"E7F7FFFF FFFFFFFF F8EAD9C3 AC957B5F 3F1B0702 00000000 00000000 05112135"
		$"485A6A77 828C959E A9B5C0C9 D1D7DCE2 E6E9E9E7 E4E2E1E2 E5E9E9E5 D6BC8F4D"
		$"210B0000 00000000 00000000 0A1E3A60 82A2BED6 E9F7FFFF FFFFFAF0 E0CABCB4"
		$"B1B3B2B0 ADABA7A3 9B91877E 74685A48 382A2220 2229323D 47515E6E 8092A1AE"
		$"BED0E2F5 FFFFFFFF FFFFF8EB DAC6B19B 83674826 0F050000 00000000 00000208"
		$"152B4157 69778289 939FACB9 C2C6CBD0 D8E2E9EE EEEAE5DF DDDDE1E8 EAE6D6BA"
		$"863C1105 00000000 00000000 00000C25 446888A2 BBD3E7F7 FFFFFFFF F8EADAC6"
		$"BAB7B7BA BAB6B3AF ACA8A096 8A7D6F61 52423429 221E1D21 28334254 67798793"
		$"9EA8B8CD E1F5FFFF FFFFFFFF F7E7D6C2 AE9A8164 431D0802 00000000 00000000"
		$"040E1F37 4A58636B 747F8EA1 B1BDC5C9 CED4DCE6 ECEEEDE8 E4E2E4EA EDEFECE4"
		$"D1B48138 0F050000 00000000 00000000 0F2D4E72 8EA2B7CD E1F5FFFF FFFFF5E2"
		$"D0BFB7B7 B6B6B5B5 B5B6B4AE A79D9286 77655342 3428211E 1F232A33 4052657A"
		$"494B4F50 4C48443C 36322E2E 312E2A2A 2E363F46 536B87A3 B5BED8F5 FEFFFFFF"
		$"FFFFFFFF FFFCF9F4 F2F9FCFC FBF5EDE3 D2BBA79F 9D8D786C 6765625E 5C5B5B5D"
		$"5E5C5A54 4A3F3832 2A242222 262C3035 3C3F4147 50565F6A 6D6B6A6A 67666564"
		$"65635F5F 5F5B554F 4C4C4B47 4A4F5153 53565B63 6E727171 72737575 7676777E"
		$"868D9496 96969591 8E90908D 8A827C7B 7D848C91 94969AA0 A8AEB0B1 B2B1B1B5"
		$"BFCDD3D3 D2D0CCC9 C6BFB6B0 ACAAAAA9 A69D8E84 7E787672 68676C6E 6C696A6C"
		$"6E716E68 6B717678 77787C81 888A8986 7E7A7C83 837E7D7A 76797E80 86888488"
		$"8B8C9091 9499A1A8 AAAAACB7 CBDDEDFF FFFFFFFF FFFFFFFF FFFFFFF6 E8E0DCD9"
		$"D2C9C4BF B9B5B1AA A0979492 897C7167 666A7279 766C655E 5C5F554B 473C3634"
		$"342F231C 21282C36 35262021 1E1E211C 100A0E12 141A1D1C 18191C17 16181818"
		$"1511131A 1F242C33 36363439 3A353535 383E4245 48494B51 59606972 7C8CA1B4"
		$"BED7FFFF FFFFFFFF FFFFFFFF FFFFFFFF FFF9F4F2 EDE5D9CA BBABA19F 90746053"
		$"463E3B3A 3C3E3F42 4444474D 5154575D 62656D74 73798081 899299A0 A5A8AAB1"
		$"BDC1BAB3 ADA7A5A4 9E928170 696E747D 84848B8B 8C96948E 8A888D96 9FA3A4A4"
		$"A4A7B0B0 AAADACA7 A3A1A1A0 9C947F70 6C696663 65605A5F 68768793 9A9FA2A5"
		$"BAD9EDF8 FEFBF3ED E6DACBBD B3AA9F99 968B745C 4D403735 3A444E53 585F5E60"
		$"6C798797 9E9DA0AA AFA79FA6 ABA9ACAE A59A9899 99989582 6D686055 4B38200F"
		$"09040000 00000210 1611131C 1714140F 1212141C 212B435E 7992A5AD B3CADFE8"
		$"F0EEE4E1 DED9D4CD C7C0BBB6 A9A0A19F 9480664C 3C2E2728 2F32343A 3D454A41"
		$"34281F20 2A302C22 1B1F2F45 49423E3F 42372A2B 2D27231F 1C273D3F 38373430"
		$"3A44433F 3830281F 1D1D2026 2E38424E 554A3B31 2A272829 2624282D 323B4B52"
		$"54606B6C 73746960 5E5D5B60 6D747D8B 90929A9C 9796928F 949CA5B2 BFC6C8C9"
		$"D2E6FBFF FFFBF1E6 DBD3CBC6 C4C3C3C7 CDCFD3D5 D2C9BFBA BAB8AF9F 958C7F7D"
		$"807E7F81 7D7C8083 86837C76 6F71767A 82827B78 7264657C 8C959997 928E9296"
		$"8E847C70 71777870 68797F6A 656A6E67 69727579 7D787374 71707E81 7D7A6863"
		$"6D717885 8C939C9C 9B9E9B96 928C8071 747B7470 6E6C7583 96A7B4B9 B8B6B1A4"
		$"98999593 A0AEB6BA BABABABA B9B7B6B2 A696867E 79777F85 83828484 80848D8A"
		$"7E757272 7173797E 79706E6C 7076746E 665C5650 55605F5A 585E7280 88939293"
		$"9B9DA0A7 A6A29E99 9592969E A2A6B2B8 B8B8B8B8 B7B7B6B4 B2B0ABA3 9986787A"
		$"81888C8B 8B8D90A0 AA9A8D84 7C7A7A77 767E8178 747A746E 77888F80 6E6F736F"
		$"7B7F6E78 887C7883 82777B8B 887F796E 685F5462 746F737C 77727C83 827E7566"
		$"5754616E 6D727572 77828987 82756A6A 645D5B56 50525A64 70767470 6E686568"
		$"6C6F7582 8F9CB0BD BFCEE2E8 EBEFF0ED E6DCD4CC C9C9CDD5 D9D7D5D1 CCCBC7BE"
		$"B7B1ADAD AAA49C9A 9588817D 7A7D7F77 70737671 6866686A 6C737268 696E6D70"
		$"7985918D 878F948E 827E827E 7A7C837B 6E6E7179 827F7266 5F595354 50423F46"
		$"4B596462 5E676A60 6B6F6467 6263757A 7E868C94 9BA0A6A5 9F9C9AA0 9E999C9E"
		$"9F9C9EA0 989AA3A1 9F9E9797 98A0A8A3 A4ACADAD AEAEADB0 B6B6B1AD ACABABAA"
		$"A8A5A4A0 998F8B89 85868985 78717576 787E796C 696A6A6E 716F6965 666C6D65"
		$"5C565963 68696967 686D757A 77706D6F 76787270 75787474 75788289 8A8D8D8C"
		$"8D91928A 8A8C8A8E 93969BA3 A9ABACAC AEB5C4D4 D7D7D8D3 D2D9D4C9 C8C1B1A9"
		$"AAADABA7 A5A2A2A2 9E92807A 7A76736D 6968686E 6F6F6E70 70665F63 686B726E"
		$"5D525662 6A6D6C6A 6A6C7079 75686D6E 6866686A 6C6F7479 6A636D6D 6B6B6E6B"
		$"5C5A6059 575E5856 6470747A 7E786B66 645E6061 5A5B605F 61666975 7B777678"
		$"79726862 625D5B65 66626667 656C6F6C 70757274 7C7C7873 695D5860 6D7B91A7"
		$"AFAFB0B1 B2BCC5C3 B8AFADAD ADACACAC AAA8A49E 9A9D9B97 989BA0A1 9E9A9289"
		$"878C8985 867E6F69 6C71777F 807D7B77 7A7E7D7F 7D6C6566 6976807D 777E8077"
		$"7B7C7774 6E6B666D 72676162 61626769 6C6B646A 6A5F5E5E 66727173 80837771"
		$"6A5B5F76 87857E70 646E7878 70666664 69808774 6A6A636C 808D7D6F 7C7B7281"
		$"8A817F83 82848381 7D7B7C77 75859696 979A9491 91909699 958E8D99 9C969594"
		$"92959897 97948E89 8483888D 92939394 9291908D 8C8C8785 87848790 8B848381"
		$"81827F79 76797A79 838B8884 7D7D8584 868B837F 8384858A 8C89848C 949097A0"
		$"94898887 88878483 7A809289 8792867D 88877E77 70768184 8385877F 737C8E80"
		$"727F7764 79857C89 93928A82 837A6A64 6461606A 6F7A8881 706D787D 75747266"
		$"61656669 6E6C6D71 80897E71 6E6E7482 827A797A 7677827B 6E737C7E 7E808387"
		$"82746F7A 817D7C7C 7677807C 6E6E7571 75837E79 7A798086 81797572 75818784"
		$"858C8F8E 949C988E 88868079 7C84817E 8487898E 928F8E94 9CA3A8A8 A4A1A3A7"
		$"A59F9E98 91949899 9795979B 9D9FA4A6 9E9CA3A1 9B9A9692 908E8D8A 86847A72"
		$"76767477 7877777A 7C7E7B76 74707269 6679807C 776A6166 676D7D80 746A7180"
		$"7267868E 736A7273 706E7983 80797C96 90675E6A 6C7E8770 626E7C79 73787D81"
		$"7C6E6B74 78746E6F 746D7383 8282837C 767E8B81 7A7F7875 7B807F7A 7D7C7983"
		$"8F918887 87828493 9481848B 7C787E85 88807C7E 7A838F8E 88817980 8682848A"
		$"82727789 88858C89 83888A87 89847D78 777F898E 8E8C8F94 908A888A 8E8F8784"
		$"8B8D8889 8C898A8F 897F7A78 797E8787 8C8F8887 85858A85 7E82898E 8F87878C"
		$"8F8E8784 88888382 8D8A8282 828D9084 7F7E8588 7E7C7F7E 878E8174 75818E8D"
		$"80888E7C 787B746A 68696A78 88827973 636A7B79 7A7D7F7E 786D626B 7166636D"
		$"7F887D74 72707374 7A837477 9391847B 7E807275 86837674 858E817E 827F8087"
		$"83716E6F 67676D76 817E767C 85827E83 807A7A7D 888A8081 82757583 84878983"
		$"83858586 86868B8A 8E8E8489 88797882 80818C88 7A7B8686 85878485 80767983"
		$"817C7C81 88909A93 888E8E82 80867E7D 8483818A 8D899192 888A8584 958F848F"
		$"82768381 7B84837D 80848687 867E767F 82737177 78777E85 7D747D85 7D7E7E7C"
		$"81807D7E 7F7C7C7F 79858F7A 78807B7D 78808476 7D868085 877D7676 76767A84"
		$"837F8886 7C83847A 71727473 74767C7F 7975756E 7C948E87 84796E68 717E838A"
		$"837A8686 7A7D817D 848A898B 90968D7F 82828486 77748985 7A939E84 8591847F"
		$"857A767F 7D70707E 86868580 89877B89 8C767681 84817E82 807C8485 7C76747B"
		$"888A8891 8A78808D 8C837A78 797C7D7E 827E7C88 8F8A8986 83878985 807D7E80"
		$"7B737882 7F7C8188 88818080 7C848C7C 78877D76 807B7D88 7A707F82 7E878075"
		$"7A797982 878C807A 7E7C7B7B 81817B7B 838A8583 89827782 8A858282 7D6D7180"
		$"74748486 868D8A86 8B81747B 817C7A7D 7F7A787A 7C82868C 8D847F84 88847C80"
		$"7E767B82 8A8B8387 8F87878C 7C7C8A8E 867E807B 74767F81 7F81858C 96988672"
		$"7883827E 817D7882 8D95968C 84898984 82807B7B 7E7D7F85 84838C8A 868C837E"
		$"85897D72 81878080 807F7E7E 7C7B7B78 7F8B8986 867F7D80 7F7D7D84 84787D80"
		$"7A85857C 8A8D8380 7B777E88 827D8683 7C7E8081 84838080 82888479 7D7B737B"
		$"7E747275 7C827F7C 7A7E7E80 857E7576 7377888C 887F7D84 85898A88 877D7D83"
		$"81888A7C 76848E84 7E8A8273 7E81787C 8385817B 7A77757B 7A74767C 837C7A84"
		$"857B737B 89807C82 7A7B827E 87897F7D 80878C8E 8A84827F 80837F7F 8482787A"
		$"8A888084 8177777A 7D7D8689 7C7C7F7C 84807680 89818089 84787A80 8581808A"
		$"7C728180 8389878A 817A7E82 8884767A 827A7D86 7E7A848A 86888A7D 727F7D74"
		$"858A776F 7777777F 7A768184 7B7D7D74 7A857A73 7C7A747E 81787A84 7F7C8A8D"
		$"898B8A8A 827B7C7B 7B7D7B82 85808C8C 828C8D86 84817F7A 77808480 81818386"
		$"85817A7B 81838D8C 8387827C 84847C79 81848286 847E8284 888A8689 8C817C86"
		$"89848686 86868183 84828680 848E8280 857C8189 817B8182 81817B7F 827D7B80"
		$"85848282 85888584 837B797C 7B7E7B75 7C80868A 8285877A 7E8A8B84 7E7E7E84"
		$"887D727C 85767A8C 8476797A 787A7A80 7D737A83 847D7B83 8784827E 7D7E7E7F"
		$"84838080 7B828B84 87898584 80847E76 85897A7D 8A877E80 86837C82 8680878C"
		$"8582807C 77777B7A 77828881 7E83827F 807F7E84 847C7A7E 7E7D7D80 837F7E83"
		$"847F818C 8F817B84 7D778688 80847F7D 8A877B7E 837D818C 8880817E 7A7C8086"
		$"88827F87 88838685 78798176 79858180 82827E81 89868284 7E71747B 797B837C"
		$"76858D8B 90908378 7A81807D 807E7883 89848986 7C797B84 827E847E 7B86887E"
		$"7D817F79 77808280 8485847F 7D808384 7D7E8079 7A858484 86828283 8284837C"
		$"7E7E7C7D 78838A7A 75848987 87837C7E 85837A79 817B7780 847F7D7F 848A887F"
		$"7E7B767F 857D7D82 807B7C81 827E8084 82848889 89878482 7F7C7977 7C848581"
		$"868C8883 84878E89 80837A73 7C7F7E81 80828483 8286847C 7B83827B 8386767B"
		$"8A7F8088 7D7C8079 7C888682 86898281 87838186 7E7A8181 80858686 86858587"
		$"87838282 7F80817D 7F7E7D80 7E7E7E7E 81827E7D 80807F82 86827F81 82817F80"
		$"827D7F84 7F7F8281 84858687 827E817A 7B87867F 82858080 837F7F83 7F80837F"
		$"7F7E7C7B 7B808382 85868284 837E7D7C 7C7C7B80 847E8086 84878A83 7E7D7C82"
		$"837F7E7B 7C87837E 88897E7E 827F7E80 80828781 7C82817C 7F838082 827E7B84"
		$"89858383 7F7C7D82 827C7E87 827D7E7D 80828183 85817E81 7E7E8384 82838682"
		$"7C7F827F 807E7A7E 817D7B80 827F7D7E 84848384 7F81847F 7E7F7E7E 8488857E"
		$"8087837F 81808080 7E7E8384 807F7E7B 80858684 83858580 7C797A82 86828085"
		$"827D8388 8684817D 7C7F807E 80868680 81818285 8181807E 7F82817C 7C7E7F80"
		$"7E7E7A78 7E7E7E85 83808281 7E828380 7E7E8084 827F7F83 83808386 86848488"
		$"88807E7F 7D7F817E 7D7F8387 86858581 81808082 7F7D7C7C 7F808283 82818181"
		$"8182827E 82837D85 877F8082 86868484 827F8484 8084807F 84828384 7F7D7C81"
		$"87837D7E 7D7C7D7D 7D858482 89878280 817F797C 81827C7D 827E7E84 8582807E"
		$"7E807C7C 82828280 7F807E80 85828080 82828081 82817E7D 7D7D8081 868C827C"
		$"82817E80 817E7E80 82827E7D 7E7E8387 847D7E82 807D7E81 807E7F82 80808280"
		$"81838386 87828182 81808080 807D7B81 85817F80 82878781 7B7C8385 807D7C7C"
		$"7B7F8682 7D828481 8385817C 7E838684 7F7E7F7F 7C7E807C 7B808482 8186837D"
		$"7E7F7C7D 7F7C7C82 85817F82 7F7D8084 84807F80 7E7C8082 7E7F817E 7D7E7C7E"
		$"7F7E8485 84858382 827E7B7C 7D7C7E7E 8184837E 7D7E7C80 84808082 7E7E8181"
		$"80838481 81808080 80807E7E 817E7D7E 7E7F7E80 8482807E 7C7F8280 7E81857E"
		$"7E848280 83817F7E 7E83847E 7D807E80 83818182 83828184 83807E7E 80808184"
		$"817F8281 81818180 7F7F8281 7D7E7F7F 84878481 83858789 857E7E80 8286847F"
		$"7F7F7F82 86898784 827E8083 7F7E7F7E 81848483 7F7F8080 84848485 807F807E"
		$"7E808282 81808482 81818184 82848481 8084837F 84888584 84808081 827F7D7F"
		$"7D7D8486 84828180 81807E80 807C7C81 84828080 7F828282 82818080 7F828383"
		$"85878685 8481807F 7F7F7E80 82808487 827E8182 81818386 817D7F80 80828483"
		$"82828285 8482817D 7F828283 82808082 8382807F 7F808487 87817C7E 7F808381"
		$"7E7F8082 84837E7F 7F7D8084 84827F80 84868581 81838484 7E7C7D7E 84858182"
		$"84818281 7D7E7F7E 81817D7F 817E7D80 85868685 7F7C7E7D 81868582 81818081"
		$"84868883 82827D7F 85817C7E 82828484 81818180 807D7E83 82817E7B 7E82807F"
		$"81807A7C 81807E81 83807E80 83858582 7E7F8281 82828181 807F7D7C 7F828182"
		$"81818384 85817E80 82827F7A 7C828586 83808080 8180807F 7B7C7E80 84847D80"
		$"85858280 807E7D7E 7E808080 82838081 8382807F 81848584 7E7B7D80 82858786"
		$"817F7E7F 82828081 807E7D80 84828082 807F8282 81808083 84828281 7F7E7F82"
		$"827F7F7F 7C7E8180 81818486 807C7E7F 82858688 857F7D7B 7D82807F 83847F7F"
		$"817F7F81 807C7C7D 7F82807E 80808084 84828283 807D7E80 7F807F7E 7F818281"
		$"80838483 82828483 7F7E7F82 8281807F 80818080 81808181 807E7D7D 7E7F7E7B"
		$"7C7D7C7F 82838285 84808182 8384817E 7F807F81 83848586 87868684 807F8080"
		$"7F80807D 7E808386 86848382 7F7E7F7F 80827E7E 85858182 83808083 81808180"
		$"7E848582 82848381 80808080 7F7E8083 807F827F 80847F7C 7F82807F 807F7E7E"
		$"7F818281 82848081 82808180 7D7D7E7E 7D7E7C7E 82818486 837F8080 7E7E7F7E"
		$"7E82817F 82807D80 84838380 7D7C7B7E 83837F7F 807D7C7F 83828182 8385827F"
		$"80807D7C 7E7E8081 81858786 8683807F 7F7F7A79 7E828082 85848484 82818180"
		$"807E7C7E 8182807F 82807F82 83828182 827F7F81 80807F7F 7E7E8080 82858582"
		$"82828281 7F7F7E7E 7F7D7F7F 7C7D8081 81848683 8284817C 7E808181 8284827E"
		$"7E7F7F81 83828283 83858685 84827F7E 7E7F7E7D 7E808182 83868988 85828180"
		$"8081807E 7F828384 85868483 827E7F81 82848584 85848280 82817E7E 7F7E8186"
		$"85828384 84848484 7E7B7F81 82838182 84838383 81808181 81807F80 82818281"
		$"80818283 827F8182 81817F80 81808184 85848381 7F7F8182 83817F7F 7F808284"
		$"86848284 84818184 857F7F81 80828483 82818384 83828184 827F8181 80808182"
		$"82807F80 8182817E 80828283 82808182 83828180 7F808181 81818281 81828181"
		$"83827F80 7F808283 83858481 83838281 7E7E7D7D 7E7E7F7F 80807F80 82807D7C"
		$"7E7F7F80 8082827F 7F808080 80807E7C 7E818283 82828281 80808283 83807E7E"
		$"7F7E8084 8482827F 7D7C7B7C 7D7E7F7C 7C80807E 80828282 7F7F8180 7F7F8080"
		$"8082817F 81817F81 84858484 84848282 8384827F 7E818280 81838280 7F808486"
		$"84828080 7F7D7E7E 7C7C7F80 80818281 7F818283 827F7F7F 7C7D8080 7F7F8182"
		$"7F808284 84858584 817E7E7F 8082817F 7F808283 82828484 84828080 7D7B7D7E"
		$"80828383 82818281 82827F7E 7E7D7B7C 80828482 82848282 82807E7E 80808282"
		$"81818182 82838483 82838181 83828485 82838584 82807E7D 7F7E7C7A 7A7D7C7A"
		$"797A7A7B 7F7F7E7F 80808183 84838182 86878585 86868482 7E7E8387 85827E7A"
		$"787B8084 837E7B7A 7B7D8283 807F8082 84858481 7F7F7F81 84848081 82807F81"
		$"82828080 81808182 81808081 81808284 84838280 81818081 807F7F7E 7E818281"
		$"7F7F7F7F 807E7D7E 80818182 82807F80 83838382 82838588 87868887 84838381"
		$"7E7B7979 7B7C7A7A 7A797979 7A7D8081 81818281 83868685 84858585 85838282"
		$"82807F80 817E7B7E 80818585 8483817F 80818281 7F7D7D7C 7A7C7F7E 7E7E7E7E"
		$"7E7F8283 83827F7E 80858380 82848484 86878583 817F8181 7E7D7B7A 7C7F7F7E"
		$"80807E7E 82807C7E 807F7F80 80808082 83838280 7F808182 82818282 81828381"
		$"7F7E7F82 83828282 82848484 85838183 82828483 82828182 82828180 82838280"
		$"81828181 8282807F 7E7D7D7F 82828081 83848382 8281817F 80808080 7F7F8284"
		$"85858584 84838382 8183827F 7F808281 81807E80 82807F81 807F7F80 8080807E"
		$"7D7E8081 81828485 82818284 84838280 7F808284 85858481 80828484 8484817F"
		$"81828183 83807E7E 7F808181 7F808181 80808184 83808081 80818384 82828282"
		$"82838382 81808081 82808080 80818182 83838180 80828485 827F7F80 80818282"
		$"817E7E7F 7E7D7D7D 7E818282 83817F7E 7D7D7F7F 7F808081 80818384 82808081"
		$"82828281 807E7E7F 80807F7E 7E808182 8281807F 7F7E7E7E 7F7F8081 82828080"
		$"81808081 807F7F81 817F8081 807F8081 82807E7E 7F818384 84828181 8181807F"
		$"7E7F8082 84838180 80818182 8180807F 7F808283 83828281 8180807F 80828282"
		$"8281807F 80828181 807F8182 82828484 84848281 80807F7F 80828281 807F8083"
		$"84858584 82828281 82828080 81828383 84838180 82818080 7F7E7F81 80818280"
		$"7F7F7F81 86868482 81828283 84848381 7E7D8082 82828180 80818081 82818283"
		$"82807F7F 7F808282 82828182 82818181 81818181 807F7F7F 80818282 8383817E"
		$"7D7E8080 80807F7F 80818282 82828182 83828281 80818282 82828282 82807F7F"
		$"80807F7F 81838382 80808081 81818080 81817F80 83838284 84828181 8081817F"
		$"7F818384 84828181 82828180 81818283 83818080 82828180 80808080 80808080"
		$"82838382 81818181 81828282 80808282 81818181 81818181 81818181 82818284"
		$"83828181 80808180 7F80807F 7F808080 7F808181 807E7D7E 7F808283 827F8082"
		$"81808180 81828280 8181817E 7E808281 80808080 7F7F8080 80808182 81807F7F"
		$"80807F7F 80808182 81828181 80818281 80808181 80808080 80818282 80807F7F"
		$"81818181 81807E80 8181807E 7D7E7F80 7F7F8080 80808182 81808081 81817F7D"
		$"7D7F8080 81818080 81818180 8080807F 7E80807E 7E7F8081 82828281 81818181"
		$"8180807F 8182807E 7E7E7E7E 80818181 82828283 83838280 807F7E7E 7E7F8080"
		$"81818182 82818080 8080807F 807F7E7E 7E7F7F80 82828383 83828282 82828281"
		$"807F8080 81828180 81818283 8381807F 7F808181 82828181 82818182 81807F7F"
		$"7E7E7E7F 7F7F7F80 81828382 81818180 80818181 81818282 83838282 81808080"
		$"807E7D7E 7F808181 80808081 81828484 83828383 81818181 80807F7E 7E7F8181"
		$"82818282 8281807F 7F7F8081 81818282 83848383 83828181 80808080 81808080"
		$"80808182 82828282 82828281 80808080 81818181 807F8082 82818181 80808080"
		$"80818080 80807F80 80818181 81818080 81818080 80808080 81818282 81818080"
		$"80808080 80808080 80808080 7F808080 81818080 80808080 80808181 81818282"
		$"82828282 81808081 80807F80 80808181 81808080 8181807F 7E7E7F7F 7F808080"
		$"80818181 81818181 81808080 80808080 7F7F8080 80818181 80818181 81818181"
		$"81818180 80808180 80808080 8080807F 80808181 80808080 80808080 80808081"
		$"81828180 80808080 80807F7E 7F808080 80808080 80808080 80808080 80808080"
		$"81818181 81818080 807F7F7F 80808080 80808080 80808080 80808080 80808080"
		$"7F7E7E7F 80807F7E 7E7F8080 8180807F 7F808080 80808080 80818282 80808081"
		$"81808080 807F7F80 80818080 80808180 80808080 80808081 80808080 80808080"
		$"8080807F 7F808080 80808080 81818181 80818181 81808080 80808080 7F7F7F80"
		$"80808080 80808080 80818181 80808080 80808080 80808081 81818180 80808080"
		$"80808080 80818181 81818181 81818080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 807F7F7F 7F7F7F7F 7F7F7F7F 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808181 81818080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
		$"80808080 80808080 80808080 80808080 80808080 80808080 80808080 80808080"
	}
};

